#!/usr/bin/env python
# encoding: utf-8
r"""
Riemann solvers for the shallow water equations.

The available solvers are:
 * Roe - Use Roe averages to caluclate the solution to the Riemann problem
 * HLL - Use a HLL solver
 * Exact - Use a newton iteration to calculate the exact solution to the
        Riemann problem

.. math::
    q_t + f(q)_x = 0

where

.. math::
    q(x,t) = \left [ \begin{array}{c} h \\ h u \end{array} \right ],

the flux function is

.. math::
    f(q) = \left [ \begin{array}{c} h u \\ hu^2 + 1/2 g h^2 \end{array}\right ].

and :math:`h` is the water column height, :math:`u` the velocity and :math:`g`
is the gravitational acceleration.
"""

import numpy as np

num_eqn = 2
num_waves = 2


def shallow_roe_1D(q_l, q_r, aux_l, aux_r, problem_data):
    r"""
    Roe shallow water solver in 1d::

        ubar = (sqrt(u_l) + sqrt(u_r)) / (sqrt(h_l) + sqrt(h_r))
        cbar = sqrt( 0.5 * g * (h_l + h_r))

        W_1 = |      1      |  s_1 = ubar - cbar
              | ubar - cbar |

        W_2 = |      1      |  s_1 = ubar + cbar
              | ubar + cbar |

        a1 = 0.5 * ( - delta_hu + (ubar + cbar) * delta_h ) / cbar
        a2 = 0.5 * (   delta_hu - (ubar - cbar) * delta_h ) / cbar

    *problem_data* should contain:
     - *g* - (float) Gravitational constant
     - *efix* - (bool) Boolean as to whether a entropy fix should be used, if
       not present, false is assumed

    :Version: 1.0 (2009-02-05)
    """

    # Array shapes
    num_rp = q_l.shape[1]

    # Output arrays
    wave = np.empty( (num_eqn, num_waves, num_rp) )
    s = np.zeros( (num_waves, num_rp) )
    amdq = np.zeros( (num_eqn, num_rp) )
    apdq = np.zeros( (num_eqn, num_rp) )

    # Compute roe-averaged quantities
    ubar = ( (q_l[1,:]/np.sqrt(q_l[0,:]) + q_r[1,:]/np.sqrt(q_r[0,:])) /
             (np.sqrt(q_l[0,:]) + np.sqrt(q_r[0,:])) )
    cbar = np.sqrt(0.5 * problem_data['grav'] * (q_l[0,:] + q_r[0,:]))

    # Compute Flux structure
    delta = q_r - q_l
    a1 = 0.5 * (-delta[1,:] + (ubar + cbar) * delta[0,:]) / cbar
    a2 = 0.5 * ( delta[1,:] - (ubar - cbar) * delta[0,:]) / cbar

    # Compute each family of waves
    wave[0,0,:] = a1
    wave[1,0,:] = a1 * (ubar - cbar)
    s[0,:] = ubar - cbar

    wave[0,1,:] = a2
    wave[1,1,:] = a2 * (ubar + cbar)
    s[1,:] = ubar + cbar

    if problem_data['efix']:
        raise NotImplementedError("Entropy fix has not been implemented.")
    else:
        s_index = np.zeros((2,num_rp))
        for m in range(num_eqn):
            for mw in range(num_waves):
                s_index[0,:] = s[mw,:]
                amdq[m,:] += np.min(s_index,axis=0) * wave[m,mw,:]
                apdq[m,:] += np.max(s_index,axis=0) * wave[m,mw,:]

    return wave, s, amdq, apdq

def shallow_hll_1D(q_l,q_r,aux_l,aux_r,problem_data):
    r"""
    HLL shallow water solver ::


        W_1 = Q_hat - Q_l    s_1 = min(u_l-c_l,u_l+c_l,lambda_roe_1,lambda_roe_2)
        W_2 = Q_r - Q_hat    s_2 = max(u_r-c_r,u_r+c_r,lambda_roe_1,lambda_roe_2)

        Q_hat = ( f(q_r) - f(q_l) - s_2 * q_r + s_1 * q_l ) / (s_1 - s_2)

    *problem_data* should contain:
     - *g* - (float) Gravitational constant

    :Version: 1.0 (2009-02-05)
    """
    # Array shapes
    num_rp = q_l.shape[1]
    num_eqn = 2
    num_waves = 2

    # Output arrays
    wave = np.empty( (num_eqn, num_waves, num_rp) )
    s = np.empty( (num_waves, num_rp) )
    amdq = np.zeros( (num_eqn, num_rp) )
    apdq = np.zeros( (num_eqn, num_rp) )

    # Compute Roe and right and left speeds
    ubar = ( (q_l[1,:]/np.sqrt(q_l[0,:]) + q_r[1,:]/np.sqrt(q_r[0,:])) /
        (np.sqrt(q_l[0,:]) + np.sqrt(q_r[0,:])) )
    cbar = np.sqrt(0.5 * problem_data['grav'] * (q_l[0,:] + q_r[0,:]))
    u_r = q_r[1,:] / q_r[0,:]
    c_r = np.sqrt(problem_data['grav'] * q_r[0,:])
    u_l = q_l[1,:] / q_l[0,:]
    c_l = np.sqrt(problem_data['grav'] * q_l[0,:])

    # Compute Einfeldt speeds
    s_index = np.empty((4,num_rp))
    s_index[0,:] = ubar+cbar
    s_index[1,:] = ubar-cbar
    s_index[2,:] = u_l + c_l
    s_index[3,:] = u_l - c_l
    s[0,:] = np.min(s_index,axis=0)
    s_index[2,:] = u_r + c_r
    s_index[3,:] = u_r - c_r
    s[1,:] = np.max(s_index,axis=0)

    # Compute middle state
    q_hat = np.empty((2,num_rp))
    q_hat[0,:] = ((q_r[1,:] - q_l[1,:] - s[1,:] * q_r[0,:]
                            + s[0,:] * q_l[0,:]) / (s[0,:] - s[1,:]))
    q_hat[1,:] = ((q_r[1,:]**2/q_r[0,:] + 0.5 * problem_data['grav'] * q_r[0,:]**2
                - (q_l[1,:]**2/q_l[0,:] + 0.5 * problem_data['grav'] * q_l[0,:]**2)
                - s[1,:] * q_r[1,:] + s[0,:] * q_l[1,:]) / (s[0,:] - s[1,:]))

    # Compute each family of waves
    wave[:,0,:] = q_hat - q_l
    wave[:,1,:] = q_r - q_hat

    # Compute variations
    s_index = np.zeros((2,num_rp))
    for m in range(num_eqn):
        for mw in range(num_waves):
            s_index[0,:] = s[mw,:]
            amdq[m,:] += np.min(s_index,axis=0) * wave[m,mw,:]
            apdq[m,:] += np.max(s_index,axis=0) * wave[m,mw,:]

    return wave, s, amdq, apdq


def shallow_fwave_1d(q_l, q_r, aux_l, aux_r, problem_data):
    r"""Shallow water Riemann solver using fwaves

    Also includes support for bathymetry but be wary if you think you might have
    dry states as this has not been tested.

    *problem_data* should contain:
     - *grav* - (float) Gravitational constant
     - *dry_tolerance* - (float) Set velocities to zero if h is below this
       tolerance.
     - *sea_level* - (float) Datum from which the dry-state is calculated.

    :Version: 1.0 (2014-09-05)
    :Version: 2.0 (2017-03-07)
    """

    g = problem_data['grav']
    dry_tolerance = problem_data['dry_tolerance']
    sea_level = problem_data['sea_level']

    num_rp = q_l.shape[1]
    num_eqn = 2
    num_waves = 2

    # Output arrays
    fwave = np.empty( (num_eqn, num_waves, num_rp) )
    s = np.empty( (num_waves, num_rp) )
    amdq = np.zeros( (num_eqn, num_rp) )
    apdq = np.zeros( (num_eqn, num_rp) )

    # Extract state
    u_l = np.where(q_l[0, :] > dry_tolerance,
                   q_l[1, :] / q_l[0, :], 0.0)
    u_r = np.where(q_r[0, :] > dry_tolerance,
                   q_r[1, :] / q_r[0, :], 0.0)
    phi_l = q_l[0, :] * u_l**2 + 0.5 * g * q_l[0, :]**2
    phi_r = q_r[0, :] * u_r**2 + 0.5 * g * q_r[0, :]**2
    h_bar = 0.5 * (q_l[0, :] + q_r[0, :])

    # Speeds
    u_hat = (np.sqrt(g * q_l[0, :]) * u_l + np.sqrt(g * q_r[0, :]) * u_r)      \
            / (np.sqrt(g * q_l[0, :]) + np.sqrt(g * q_r[0, :]))
    c_hat = np.sqrt(g * h_bar)
    s[0, :] = np.amin(np.vstack((u_l - np.sqrt(g * q_l[0, :]),
                                 u_hat - c_hat)), axis=0)
    s[1, :] = np.amax(np.vstack((u_r + np.sqrt(g * q_r[0, :]),
                                 u_hat + c_hat)), axis=0)

    delta1 = q_r[1, :] - q_l[1, :]
    delta2 = phi_r - phi_l + g * h_bar * (aux_r[0, :] - aux_l[0, :])

    beta1 = (s[1, :] * delta1 - delta2) / (s[1, :] - s[0, :])
    beta2 = (delta2 - s[0, :] * delta1) / (s[1, :] - s[0, :])

    fwave[0, 0, :] = beta1
    fwave[1, 0, :] = beta1 * s[0, :]
    fwave[0, 1, :] = beta2
    fwave[1, 1, :] = beta2 * s[1, :]

    for m in range(num_eqn):
        for mw in range(num_waves):
            amdq[m, :] += (s[mw, :] < 0.0) * fwave[m, mw, :]
            apdq[m, :] += (s[mw, :] > 0.0) * fwave[m, mw, :]

            amdq[m, :] += (s[mw, :] == 0.0) * fwave[m, mw, :] * 0.5
            apdq[m, :] += (s[mw, :] == 0.0) * fwave[m, mw, :] * 0.5

    return fwave, s, amdq, apdq


def shallow_exact_1D(q_l, q_r, aux_l, aux_r, problem_data):
    r"""
    Exact shallow water Riemann solver

    .. warning::
        This solver has not been implemented.

    """
    raise NotImplementedError("The exact swe solver has not been implemented.")



"""
Module Iplot for interactive plotting.

For more instructions see the Usage notes in class Iplot below. 

For options during looping type:
  >>> ip = iplot()
  >>> ip.plotloop()
  IPLOT> help

"""

import cmd, os
import matplotlib
matplotlib.rc('text', usetex=False)
matplotlib.interactive(True)
import matplotlib.pyplot as plt

#------------------------
class Iplot(cmd.Cmd):
#------------------------
    """
    Class for interactively stepping through plots.
    This is an abstraction of the Iplotclaw class, meant to
    be flexible enough to deal with output from any simulation code
    (or even experimental data).

    Usage:
    ------
    >>> from clawpack.visclaw.iplot import Iplot
    >>> ip = Iplot()              # new instantiation
    >>> ip.plotloop()             # to start looping
    IPLOT > help                  # for list of available commands
    IPLOT > q                     # to quit looping and return to python
    >>> ip.plotloop()             # to restart looping at previous frame

    Arguments:
    ----------
    - load_frame: a function with the following signature:

        frame = load_frame(frameno)

        where frameno is an integer specifying the data to be loaded
        and frame is the loaded data.

    - plot_frame: a function with the following signature:

        plot_frame(frame)

        where frame is the object returned by load_frame.

    Other arguments of Iplot rarely need to be changed:
       completekey='tab', stdin=None, stdout=None
    """

    # initialization:
    # ---------------

    prompt = 'IPLOT > '
    lastcmd = 'n'             # initialize so <return> advances frame'

    def __init__(self, load_frame, plot_frame, \
                 completekey='tab', stdin=None, stdout=None):
        """Instantiate a line-oriented interpreter framework.

        The function load_frame should have the following signature:

        frame = load_frame(frameno)

        where frameno is an integer specifying the data to be loaded
        and frame is the loaded data.

        The function plot_frame should have the following signature:

        plot_frame(frame)

        where frame is the object returned by load_frame.

        The optional argument 'completekey' is the readline name of a
        completion key; it defaults to the Tab key. If completekey is
        not None and the readline module is available, command completion
        is done automatically. The optional arguments stdin and stdout
        specify alternate input and output file objects; if not specified,
        sys.stdin and sys.stdout are used.
        """

        import sys

        if stdin is not None:
            self.stdin = stdin
        else:
            self.stdin = sys.stdin
        if stdout is not None:
            self.stdout = stdout
        else:
            self.stdout = sys.stdout
        self.cmdqueue = []
        self.completekey = completekey
 
        self.load_frame = load_frame
        self.plot_frame = plot_frame

        self.restart = False
        self.prevframeno = 0

        self.frames = {}

    def plot_and_cache(self,frameno):
        if frameno not in list(self.frames.keys()):
            try:
                frame = self.load_frame(self.frameno)
                self.frames[str(frameno)] = frame
                self.plot_frame(frame)
            except IOError:
                print("Swallowing IOError to avoid crashing in interactive mode.")
 

    def preloop(self):

        print('\nInteractive plotting... ')
        print('Type ? at IPLOT prompt for list of commands')

        startframeno = input('\n    Start at which frame [default=%i] ? '\
                                % self.prevframeno)

        makeplot = True 

        if startframeno == '':
            self.frameno = self.prevframeno
            if self.restart:
                replot = input('    Replot data for frame %s [no] ? ' \
                                % self.frameno)

                if replot not in ('y','yes','Y'):
                    makeplot = False

                if makeplot:
                    reload = input('    Reload data for frame %s [no] ? ' \
                                    % self.frameno)
                    if reload in ('y','yes','Y'):
                        self.frames.pop(str(self.frameno))
        else:
            try:
                self.frameno = int(startframeno)
            except ValueError:
                print('\n    *** Error: frameno must be an integer, resetting to 0')
                self.frameno = 0

        if makeplot:
            self.plot_and_cache(self.frameno)

        self.lastcmd = 'n'
        self.restart = True 

    # end of initialization 
    # ---------------------

    def postloop(self):
        self.prevframeno = self.frameno


    # Commands that can be typed at the IPLOT> prompt:
    # ---------------------------------------------------

    # help command:
    # -------------
    def help_help(self):
        print('print this list of valid commands\n')

    # next frame:
    # -----------
    def do_n(self, rest):
        self.frameno = self.frameno+1
        self.plot_and_cache(self.frameno)
    def help_n(self):
        print('n: advance to next frame\n')

    # previous frame:
    # ---------------
    def do_p(self, rest):
        self.frameno = max(self.frameno-1, 0)
        self.plot_and_cache(self.frameno)
    def help_p(self):
        print('p: go back to previous frame\n')

    # jump to arbitrary frame:
    # ------------------------
    def do_j(self, rest):
        try:
            newframeno = int(rest)
        except:
            newframeno = input('\n    Jump to which frame? ')
        if newframeno == 'n': 
            self.do_n(rest)
            self.lastcmd = 'n'
        elif newframeno == 'p': 
            self.do_p(rest)
            self.lastcmd = 'p'
        else:
            try:
                newframeno = int(newframeno)
            except ValueError:
                print('\n    *** Error: frameno must be an integer, n, or p')
            self.frameno = newframeno
            self.plot_and_cache(self.frameno)
    def help_j(self):
        print('j N: jump to frame N\n')
        print('j:   jump to some other frame (will prompt for N)\n')

    # redraw frame:
    # -------------
    def do_r(self, rest):
        self.plot_and_cache(self.frameno)
    def help_r(self):
        print('r: redraw the current frame,  rr: reload and redraw\n')

    def do_rr(self, rest):
        try:
            self.frames.pop(str(self.frameno))
            print('Cleared data for frame ',self.frameno)
        except KeyError:
           print('No frame data to clear for frame ',self.frameno)
        self.plot_and_cache(self.frameno)
    def help_rr(self):
        print('r: redraw the current frame,  rr: reload and redraw\n')

    # clearframes
    # ---------
    def do_clearframes(self, rest):
        if rest=='':
            self.frames.clear()
            print('Cleared all frames')
        else:
            for framestr in rest.split():
                try:
                    frameno = int(framestr)
                except ValueError:
                    print('Error in clearframes: unrecognized input')
                popped_frame = self.frames.pop(str(frameno),None)
                if popped_frame is None:
                   print('No frame data to clear for frame ',frameno)
                else:
                   print('Cleared data for frame ',frameno)

    def help_clearframes(self):
        print('clearframes: delete frame data from cache to replot')
        print('    use if you have rerun the code and want to plot the')
        print('    latest results')
        print('          clearframes framenos  clears one or more frames')
        print('          clearframes           clears all frames')


    # save
    # ---------
    def do_save(self, rest):
        rest = rest.split()
        if len(rest)==2:
            try:
                figno = int(rest[0])
            except ValueError:
                print("*** Expected figure number, got: ",rest[0])
            fname = rest[1]
            plt.figure(figno)
            try:
                plt.savefig(fname)
                print("Saved figure number %s to file %s" % (figno,fname))
            except ValueError:
                print("Don't put quotes around the filename.")
        else:
            print("*** save requires two arguments: figno, fname")
            print("*** got: ",rest)

    def help_save(self):
        print('save figno fname: save figure figno to file fname using savefig.')


    # print working directory:
    # ------------------------
    def do_pwd(self, rest):
        print('  now in directory: ',os.getcwd())
        print('  data from outdir: ',self.plotdata.outdir)
    def help_pwd(self):
        print('pwd: print current working directory and outdir')
        print('     fort.* files in outdir provide frame data\n')


    # print figure to a file:
    # -----------------------
    def do_print(self, rest):
        fname = rest
        for figno in plt.get_fignums():
            if len(fname)>0:
                import string
                # doesn't work properly!
                plt.figure(figno)
                name = fname.split('.')[0]+string.zfill(figno,4)+'.'+fname.split('.')[1]
                plt.savefig(name)
            else:
                print('You must specify a file name.')

    def help_print(self):
        print('print: print all figures for this frame to files of the form')
        print('      frame000NfigJ.png')
        print('To print a single figure or with different style, try e.g.')
        print('     IPLOT > q')
        print('     figure(2)')
        print('     savefig("myname.jpg")\n')
        

    # use vi e.g. to edit setplot.py:
    # -------------------------------
    def do_vi(self, rest):
        exitcode = os.system('vi %s' % rest)
        if exitcode != 0:
            print('*** System vi command failed.  Try "help edit"')

    def help_vi(self):
        print('Edit file using vi, for example to change the plot parameters:')
        print('    IPLOT> vi setplot.py ')
        print('    IPLOT> resetplot ')
        print('See also "help edit" for use of other editors.\n')
        

    # edit a file using editor specified by environment variable EDITOR:
    # -----------------------------------------------------------------
    def do_edit(self, rest):
        try:
            editor = os.environ['EDITOR']
            eval("os.system('%s %s')" % (editor,rest))
        except:
            print('*** Environment variable EDITOR not set... ')
            print('*** Type "help edit" for more info')

    def help_edit(self):
        print('Edit file, for example to change the plot parameters:')
        print('    IPLOT> edit setplot.py ')
        print('    IPLOT> resetplot ')
        print('Specify the editor by setting environment variable EDITOR')
        print('  before starting Python shell.')
        print('If you want to use vi, see also "help vi".\n')

        
    # quit commands:
    # --------------
    def do_quit(self, rest):
        print('quitting...')
        return True
    def help_quit(self):
        print('q or quit: terminates the command loop\n')
        
    def do_q(self, rest):
        print('quitting...')
        return True
    def help_q(self):
        print('q or quit: terminates the command loop\n')
        
    def do_k(self, rest):
        print('quitting...')
        return True
    def help_k(self):
        print('k: terminates the command loop\n')
        
    def do_EOF(self, rest):
        print("quitting...")
        return True
    def help_EOF(self):
        print("Terminates the command loop\n")
        
    # alias plotloop = cmdloop:
    # -------------------------
    def plotloop(self):
        self.cmdloop()



r"""Basic functions from the old version of topotools for testing purposes."""

import numpy
import re

import nose

def topo1writer(outfile, topo, xlower, xupper, ylower, yupper, nxpoints,
                             nypoints):
    fout=open(outfile, 'w')
    dx = float(xupper-xlower)/(nxpoints-1)
    dy = float(yupper-ylower)/(nypoints-1)

    x = numpy.linspace(xlower,xupper,nxpoints)
    y = numpy.linspace(ylower,yupper,nypoints)
    X,Y = numpy.meshgrid(x,y)
    Z = topo(X,Y).T
    

    for jj in range(0,nypoints):
        y = yupper - jj*dy
        for i in range(0,nxpoints):
            x =  xlower + i*dx
            j = nypoints - 1 - jj
            z = Z[i,j]
            fout.write("%22.15e  %22.15e  %22.15e\n" % (x,y,z)) 

    fout.close

def topoheaderread(inputfile, closefile=True):
    topoheader={'ncols':0,'nrows':0,'xll':0.0,'yll':0.0,'cellsize':0.0,'nodata_value':0}
    keylist=list(topoheader.keys())

    keymap = {'ncols':'ncols','nrows':'nrows','xll':'xll','yll':'yll','cellsize':'cellsize','nodata_value':'nodata_value', \
    'xllcenter':'xll','yllcenter':'yll','xllcorner':'xll','yllcorner':'yll'}

    fid=open(inputfile,'r')
    keyleft=len(keylist)
    while keyleft> 0 :
        line=fid.readline().split()
        if line:
            if line[0].lower() in list(keymap.keys()):
                topoheader[keymap[line[0].lower()]]= convertd2e(line[1])
                keyleft=keyleft-1
            if line[1].lower() in list(keymap.keys()):
                topoheader[keymap[line[1].lower()]]= convertd2e(line[0])
                keyleft=keyleft-1

    #check if passes convert strings values to numeric
    for key in keylist :
        if not key in topoheader:
            print(('ERROR: topoheader not fully specified in %s' % (inputfile)))
            exit
        else:
            if '.' in topoheader[key] or 'nan' in topoheader[key].lower() or 'e' in topoheader[key].lower():
                topoheader[key]=float(topoheader[key])
            else:
                topoheader[key]=int(topoheader[key])


    if closefile:
        fid.close()
        return topoheader
    else:
        return (fid,topoheader)


def datafile2array(datafile=" ",sep=None, dtype="float",skiplines=0, \
skipfirstcols=0, skiplastcols=0):
    fid=open(datafile)
    data=fid.readlines()
    fid.close()

    dataarray=[]
    for row in range(skiplines,len(data)):
        data[row]=convertd2e(data[row])
        data[row]=data[row].split(sep)
        if data[row]!=[]:
            if dtype!=" ":
                for col in range(skipfirstcols,len(data[row])-skiplastcols) :
                    if dtype=="float":
                        data[row][col]=float(data[row][col])
                    elif dtype=="int":
                        data[row][col]=int(data[row][col])
            if dataarray!=[]: 
                if len(data[row])-skipfirstcols-skiplastcols==len(dataarray[0]):
                    dataarray.append(data[row][skipfirstcols:len(data[row])-skiplastcols])
            else:
                dataarray.append(data[row][skipfirstcols:len(data[row])-skiplastcols])
    
    dataarray=numpy.array(dataarray)
    return dataarray


def topofile2griddata(inputfile, topotype=2):

    if topotype>1:
        (fin,topoheader)=topoheaderread(inputfile,closefile=False)
        zdata=fin.readlines()
        fin.close()
        for row in range(len(zdata)):
            zdata[row]=convertd2e(zdata[row])
            zdata[row]=zdata[row].split()
            for col in range(len(zdata[row])) :
                zdata[row][col]=float(zdata[row][col])

        Z=numpy.array(zdata)
        Z=numpy.reshape(Z,(topoheader['nrows'],topoheader['ncols']))

        xlower=topoheader['xll']
        xupper=xlower+ topoheader['cellsize']*(topoheader['ncols']-1)

        ylower = topoheader['yll']
        yupper = ylower+ topoheader['cellsize']*(topoheader['nrows']-1)

        x=numpy.linspace(xlower,xupper,topoheader['ncols'])
        y=numpy.linspace(ylower,yupper,topoheader['nrows'])
        [X,Y]=numpy.meshgrid(x,y)
        Y=numpy.flipud(Y)
    else:
        a=datafile2array(inputfile)
        xdiff=numpy.diff(a[:,0])
        #inddiff=pylab.find(xdiff<0)  
        inddiff = numpy.nonzero(xdiff<0)[0]  # rewrite above line without find
        xlength=inddiff[0]+1
        ylength=len(a[:,0])//xlength
        x=a[:,0]
        y=a[:,1]
        z=a[:,2]

        X=numpy.reshape(x,(ylength,xlength))
        Y=numpy.reshape(y,(ylength,xlength))
        Z=numpy.reshape(z,(ylength,xlength))

    return X,Y,Z

def convertd2e (numberstring=" "):
    Dd=re.compile("[Dd]")
    newstring=Dd.sub("e",numberstring)
    return newstring



#!/usr/bin/env python

r"""
GeoClaw util Module  `$CLAW/geoclaw/src/python/geoclaw/util.py`

Module provides provides utility functions.


:Functions:

 - dms2decimal - Convert (degrees, minutes, seconds) to decimal degrees
 - dist_meters2latlong - Convert dx, dy distance in meters to degrees
 - dist_latlong2meters - Convert dx, dy distance in degrees to meters
 - haversine - Calculate the haversine based great circle distance
 - inv_haversine - Inverts the haversine distance
 - bearing - Compute the bearing from on location to another
 - gctransect - Compute a set of points on the great circle between two points
 - fetch_noaa_tide_data - Fetches water levels and tide predictions
"""

import io
import os
import os.path

import numpy
from urllib.parse import urlencode
from urllib.request import urlopen

# ==============================================================================
#  Constants
# ==============================================================================
from clawpack.geoclaw.data import Rearth, DEG2RAD, RAD2DEG, LAT2METER

NOAA_API_URL = 'https://tidesandcurrents.noaa.gov/api/datagetter'

# ==============================================================================
#  Functions for calculating Distances
# ==============================================================================
def dms2decimal(d,m,s,coord='N'):
    r"""Convert coordinates in (degrees, minutes, seconds) to decimal form.  
    
    If coord == 'S' or coord == 'W' then value is negated too.

    :Example: 

        >>> topotools.dms2decimal(7,30,36,'W')
        -7.51

    (Note that you might want to add 360 to resulting W coordinate
    if using E coordinates everywhere in a computation spanning date line.)

    :Returns: float

    """

    deg = d + m / 60.0 + s / 3600.0
    if coord in ['S','W']:
        deg = -deg

    return deg


def dist_latlong2meters(dx, dy, latitude=0.0):
    """Convert distance from degrees longitude-latitude to meters.

    Takes the distance described by *dx* and *dy* in degrees and converts it into
    distances in meters.

    returns (float, float) 

    """

    dym = Rearth * DEG2RAD * dy
    dxm = Rearth * numpy.cos(latitude * DEG2RAD) * dx * DEG2RAD

    return dxm,dym


def dist_meters2latlong(dx, dy, latitude=0.0):
    """Convert distance from meters to degrees of longitude-latitude.

    Takes the distance described by *dx* and *dy* in meters and converts it into
    distances in the longitudinal and latitudinal directions in degrees.  

    returns (float, float)

    """

    dxd = dx / (Rearth * numpy.cos(latitude * DEG2RAD)) * RAD2DEG
    dyd = dy * RAD2DEG / Rearth

    return dxd, dyd


def haversine(x0, y0, x1=None, y1=None, units='degrees'):

    """
    x0,y0 is assumed to be a point (or an array with the same shapes as x1,y1)
    x1,y1 is a point or two arrays of points (of the same dimension)
    returns array with same shape as x1 and y1 containing distance of each point
    from (x0,y0).

    For backward compatibility, also allows x0,y0 to be 2-tuples specifying
    two points, but this is not suggested since the notation is not consistent.
    """
    
    if x1 is None:
        # for backward compatibility, assume in this case that x0 and y0 
        # are tuples for the two desired points:
        assert len(x0)==len(y0)==2, "*** Unexpected input"
        x1,y1 = y0
        x0,y0 = x0

    if units == 'degrees':
        # convert to radians:
        x0 = x0*DEG2RAD
        y0 = y0*DEG2RAD
        x1 = x1*DEG2RAD
        y1 = y1*DEG2RAD

    dx = x1 - x0
    dy = y1 - y0

    # angle subtended by two points, using Haversine formula:
    dsigma = 2.0 * numpy.arcsin( numpy.sqrt( numpy.sin(0.5 * dy)**2   \
            + numpy.cos(y0) * numpy.cos(y1) * numpy.sin(0.5 * dx)**2))

    return Rearth * dsigma


def inv_haversine(d,x1,y1,y2,Rsphere=Rearth,units='degrees'):
    r"""Invert the Haversine function to find dx given a distance and point.


    Invert the haversine function to find dx given distance d and (x1,y1) and y2.
    The corresponding x2 can be x1+dx or x1-dx.
    May return NaN if no solution.
    """

    if units=='degrees':
        # convert to radians:
        x1 = x1 * RAD2DEG
        y1 = y1 * RAD2DEG
        y2 = y2 * RAD2DEG
    elif units != 'radians':
        raise Exception("unrecognized units")
    dsigma = d / Rsphere
    cos_dsigma = (numpy.cos(dsigma) - numpy.sin(y1)*numpy.sin(y2)) / (numpy.cos(y1)*numpy.cos(y2))
    dx = numpy.arccos(cos_dsigma)
    if units=='degrees':
        dx = dx * RAD2DEG
    return dx


def bearing(x0, y0, x1, y1, units='degrees', bearing_units='degrees'):

    """
    Compute the bearing from (x0,y0) to (x1,y1), i.e., the angle clockwise from
    due North of the great circle path from point 0 to 1.  

    The value returned is thus between 0 and 360 if bearing_units='degrees',
    or between 0 and 2*pi if bearing_units='radians'.

    Note: If using this to initialize a radially-symmetric 2d velocity on the
    sphere based on a radial velocity U(r), symmetric about (x0, y0), set:
        # lat-long assumed to be in degrees, r in meters
        r = haversine(x0,y0,x,y)
        beta = bearing(x0,y0,x,y,bearing_units='radians')
        u = U(r) * sin(beta)  # beta measured from North!
        v = U(r) * cos(beta)
    """
    from math import atan2, degrees, radians

    if units == 'degrees':
        # convert to radians:
        x0 = x0*DEG2RAD
        y0 = y0*DEG2RAD
        x1 = x1*DEG2RAD
        y1 = y1*DEG2RAD
    elif units != 'radians':
        raise Exception("unrecognized units")

    dx = x1 - x0
    dy = y1 - y0
    xx = numpy.cos(y0)*numpy.sin(y1) - numpy.sin(y0)*numpy.cos(y1)*numpy.cos(dx)
    yy = numpy.sin(dx) * numpy.cos(y1)
    b = atan2(yy, xx)   # in radians from North (between -pi and pi)

    beta = (degrees(b) + 360) % 360  # convert to degrees clockwise from North

    if bearing_units == 'radians':
        beta = radians(beta)  # convert to radians clockwise (0 to 2*pi)

    elif bearing_units != 'degrees':
        raise Exception("unrecognized bearing_units")

    return beta
    
def gctransect(x1,y1,x2,y2,npts,coords='W',units='degrees',Rearth=Rearth):
    """
    Given (longitude,latitude) pairs (x1,y1), (x2,y2) and npts
    Compute (x,y) for npts equally spaced points on the great circle connecting
    them.
    
    If coords='W' the points will all have -2*pi < x <= 0.
    If coords='E' the points will all have -0 <= x < 2*pi.
    With continuity at the date line x = \pm pi.

    Sample usage for 50 points on great circle from Tohoku to Crescent City:

        from clawpack.geoclaw import util,kmltools
        xtrans,ytrans = util.gctransect(142,37,-124.2,41.74,50,'W')
        kmltools.transect2kml(xtrans,ytrans)
        # then open transect.kml to view on Google Earth

    Based in part on formulas in
         https://math.stackexchange.com/questions/1783746

    """
    from numpy import pi,sin,cos,arcsin,sqrt,arctan2,array,dot,zeros,linspace
    
    assert coords in ['E','W'], '*** coords must be E or W'
    assert units in ['radians','degrees'], '*** units must be degrees or radians'
    
    d2r = pi/180
    if units=='degrees':
        # convert to radians:
        x1 = x1 * d2r
        y1 = y1 * d2r
        x2 = x2 * d2r
        y2 = y2 * d2r    

    # compute Cartesian coordinates in 3D:
    V1 = array([cos(x1)*cos(y1), sin(x1)*cos(y1), sin(y1)])
    V2 = array([cos(x2)*cos(y2), sin(x2)*cos(y2), sin(y2)])
    d = dot(V1,V2)
    beta = sqrt(1/(1-d**2))
    alpha = -beta*d
    W = alpha*V1 + beta*V2
    yW = arcsin(W[2])
    xW = arcsin(W[1]/cos(yW))

    # compute transect points:
    t2 = arcsin(1/beta)
    if d<0: t2 = pi - t2
    t = linspace(0, t2, npts)

    xtrans = zeros(npts)
    ytrans = zeros(npts)

    for j,tj in enumerate(t):
        V = cos(tj)*V1 + sin(tj)*W
        yV = arcsin(V[2])        # latitude between -pi/2 and pi/2
        xV = arctan2(V[1],V[0])  # longitude between -pi and pi

        if coords == 'W' and V[1]>0:
            # want -2*pi < xV <= 0, continuous at -pi
            xV = xV - 2*pi
        if coords == 'E' and V[1]<0:
            # want 0 <= xV < 2*pi, continuous at pi
            xV = xV + 2*pi    
                    
        if units=='degrees':
            xV = xV/d2r
            yV = yV/d2r
        xtrans[j] = xV
        ytrans[j] = yV
        
    return xtrans, ytrans
    


def fetch_noaa_tide_data(station, begin_date, end_date, time_zone='GMT',
                         datum='STND', units='metric', cache_dir=None,
                         verbose=True):
    """Fetch water levels and tide predictions at given NOAA tide station.

    The data is returned in 6 minute intervals between the specified begin and
    end dates/times.  A complete specification of the NOAA CO-OPS API for Data
    Retrieval used to fetch the data can be found at:

        https://tidesandcurrents.noaa.gov/api/

    By default, retrieved data is cached in the geoclaw scratch directory
    located at:

        $CLAW/geoclaw/scratch

    :Required Arguments:
      - station (string): 7 character station ID
      - begin_date (datetime): start of date/time range of retrieval
      - end_date (datetime): end of date/time range of retrieval

    :Optional Arguments:
      - time_zone (string): see NOAA API documentation for possible values
      - datum (string): see NOAA API documentation for possible values
      - units (string): see NOAA API documentation for possible values
      - cache_dir (string): alternative directory to use for caching data
      - verbose (bool): whether to output informational messages

    :Returns:
      - date_time (numpy.ndarray): times corresponding to retrieved data
      - water_level (numpy.ndarray): preliminary or verified water levels
      - prediction (numpy.ndarray): tide predictions
    """
    # use geoclaw scratch directory for caching by default
    if cache_dir is None:
        if 'CLAW' not in os.environ:
            raise ValueError('CLAW environment variable not set')
        claw_dir = os.environ['CLAW']
        cache_dir = os.path.join(claw_dir, 'geoclaw', 'scratch')

    def fetch(product, expected_header, col_idx, col_types):
        noaa_params = get_noaa_params(product)
        cache_path = get_cache_path(product)

        # use cached data if available
        if os.path.exists(cache_path):
            if verbose:
                print('Using cached {} data for station {}'.format(
                    product, station))
            return parse(cache_path, col_idx, col_types, header=True)

        # otherwise, retrieve data from NOAA and cache it
        if verbose:
            print('Fetching {} data from NOAA for station {}'.format(
                product, station))
        full_url = '{}?{}'.format(NOAA_API_URL, urlencode(noaa_params))
        with urlopen(full_url) as response:
            text = response.read().decode('utf-8')
            with io.StringIO(text) as data:
                # ensure that received header is correct
                header = data.readline().strip()
                if header != expected_header or 'Error' in text:
                    # if not, response contains error message
                    raise ValueError(text)

                # if there were no errors, then cache response
                save_to_cache(cache_path, text)

                return parse(data, col_idx, col_types, header=False)

    def get_noaa_params(product):
        noaa_date_fmt = '%Y%m%d %H:%M'
        noaa_params = {
            'product': product,
            'application': 'NOS.COOPS.TAC.WL',
            'format': 'csv',
            'station': station,
            'begin_date': begin_date.strftime(noaa_date_fmt),
            'end_date': end_date.strftime(noaa_date_fmt),
            'time_zone': time_zone,
            'datum': datum,
            'units': units
        }
        return noaa_params

    def get_cache_path(product):
        cache_date_fmt = '%Y%m%d%H%M'
        dates = '{}_{}'.format(begin_date.strftime(cache_date_fmt),
                               end_date.strftime(cache_date_fmt))
        filename = '{}_{}_{}'.format(time_zone, datum, units)
        abs_cache_dir = os.path.abspath(cache_dir)
        return os.path.join(abs_cache_dir, product, str(station), dates, 
                            filename)

    def save_to_cache(cache_path, data):
        # make parent directories if they do not exist
        parent_dir = os.path.dirname(cache_path)
        if not os.path.exists(parent_dir):
            os.makedirs(parent_dir)

        # write data to cache file
        with open(cache_path, 'w') as cache_file:
            cache_file.write(data)

    def parse(data, col_idx, col_types, header):
        # read data into structured array, skipping header row if present
        a = numpy.genfromtxt(data, usecols=col_idx, dtype=col_types,
                             skip_header=int(header), delimiter=',',
                             missing_values='')

        # return tuple of columns
        return tuple(a[col] for col in a.dtype.names)

    # only need first two columns of data; first column contains date/time,
    # and second column contains corresponding value
    col_idx = (0, 1)
    col_types = 'datetime64[m], float'

    # fetch water levels and tide predictions
    try:
        date_time, water_level = fetch(
            'water_level', 'Date Time, Water Level, Sigma, O or I (for verified), F, R, L, Quality',
            col_idx, col_types)
    except:
        print('*** Fetching water_level failed, returning None')
        date_time = None
        water_level = None

    try:
        date_time2, prediction = fetch('predictions', 'Date Time, Prediction',
                                       col_idx, col_types)
        if date_time is None:
            date_time = date_time2

    except:
        print('*** Fetching prediction failed, returning None')
        date_time2 = None
        prediction = None

    # ensure that date/time ranges are the same
    if (date_time is not None) and (date_time2 is not None):
        if not numpy.array_equal(date_time, date_time2):
            raise ValueError('Received data for different times')

    return date_time, water_level, prediction



"""
Make plots for Figures 5 and 6 in the paper.
"""

from __future__ import print_function
from pylab import *
from scipy.integrate import quad
import os
from clawpack.clawutil.runclaw import runclaw

import combine_steps  

run_code = True  # set to fault if output already exists
if run_code:
    # create executable and .data files:
    os.system('make .exe')
    os.system('make data')

savefig_ext = '.png'
figdir = './figures'
os.system('mkdir -p %s' % figdir)

def save_figure(fname):
    """Save figure to figdir with desired extension"""
    full_fname = os.path.join(figdir,fname) + savefig_ext
    savefig(full_fname, bbox_inches='tight')
    print('Created %s' % full_fname)


xs = 10.e3
xlimits = [-150e3,50e3]
outdir = '_output'

if run_code:
    runclaw(xclawcmd='xgeo',outdir=outdir)   # run clawpack code


griddata = loadtxt(outdir + '/celledges.data', skiprows=1)
xgrid = griddata[:,0]
zgrid = griddata[:,1]
xcell = 0.5*(xgrid[:-1]+xgrid[1:])

figure(11, figsize=(7,3))
clf()
#plot(xgrid,zgrid,'k')
if 1:
    fill_between(xgrid,zgrid,0,color=[0.7,0.7,1])
    plot(xgrid,zgrid,'k')
    plot(xgrid,0*zgrid,'b')
    ylim(-3500,300)
#title('Bathymetry')
xticks([-xs,0,xs],['$-\epsilon$','0','$\epsilon$'])
xlim(xlimits)
ylabel('meters')

save_figure('wave_topo')


framenos = [0,5,10,20,40,70]
for frameno in framenos:
    fname = outdir + '/fort.q%s' % str(frameno).zfill(4)
    q = loadtxt(fname, skiprows=6)
    fname = outdir + '/fort.t%s' % str(frameno).zfill(4)
    t = float(open(fname).readline().split()[0])
    print('t = %g' % t)
    figure(12, figsize=(7,3))
    clf()
    plot(xcell, q[:,2], 'b')

    txs = t/xs
    if t==0:
        title('t = 0')
    else:
        title('t = %4.3f$ \epsilon$' % txs)
    xticks([-xs,0,xs],['$-\epsilon$','0','$\epsilon$'])

    hl = 3200.
    hr = 200.
    greens = (hl/hr)**(0.25)
    #print 'greens = ',greens
    #plot(current_data.x, greens*ones(current_data.x.shape),'g--')
    plot(xlimits,[greens,greens],'g--',label="$C_G$, Green's Law")
    ctrans = 2*sqrt(hl)/(sqrt(hl)+sqrt(hr))
    plot(xlimits,[ctrans,ctrans],'r--',label="$C_T$, Transmission coefficient")
    #crefl = (sqrt(hl)-sqrt(hr))/(sqrt(hl)+sqrt(hr))
    #print 'ctrans = ',ctrans
    legend(loc='lower left')

    xlim(xlimits)
    ylabel('meters')
    draw()

    fname = 'wave_%s' % str(frameno).zfill(4)
    save_figure(fname)

# make plots for Figure 6 in paper:

combine_steps.make_plot(60)
combine_steps.make_plot(70)



#!/usr/bin/env python
# encoding: utf-8
r"""
Compressible Euler flow in cylindrical symmetry
===============================================

Solve the Euler equations of compressible fluid dynamics in 2D r-z coordinates:

.. math::
    \rho_t + (\rho u)_x + (\rho v)_y & = - \rho v / r \\
    (\rho u)_t + (\rho u^2 + p)_x + (\rho uv)_y & = -\rho u v / r \\
    (\rho v)_t + (\rho uv)_x + (\rho v^2 + p)_y & = - \rho v^2 / r \\
    E_t + (u (E + p) )_x + (v (E + p))_y & = - (E + p) v / r.

Here :math:`\rho` is the density, (u,v) is the velocity, and E is the total energy.
The radial coordinate is denoted by r.

The problem involves a planar shock wave impacting a spherical low-density bubble.
The problem is 3-dimensional but has been reduced to two dimensions using 
cylindrical symmetry.

This problem demonstrates:

    - how to incorporate source (non-hyperbolic) terms using both Classic and SharpClaw solvers
    - how to impose a custom boundary condition
    - how to use the auxiliary array for spatially-varying coefficients
"""

import numpy as np
from clawpack import riemann
from clawpack.riemann.euler_5wave_2D_constants import density, x_momentum, y_momentum, \
        energy, num_eqn

gamma = 1.4 # Ratio of specific heats

x0=0.5; y0=0.; r0=0.2


def ycirc(x,ymin,ymax):
    if ((x-x0)**2)<(r0**2):
        return max(min(y0 + np.sqrt(r0**2-(x-x0)**2),ymax) - ymin,0.)
    else:
        return 0

def qinit(state,rhoin=0.1,pinf=5.):
    from scipy import integrate

    gamma1 = gamma - 1.

    grid = state.grid

    rhoout = 1.
    pout   = 1.
    pin    = 1.
    xshock = 0.2

    rinf = (gamma1 + pinf*(gamma+1.))/ ((gamma+1.) + gamma1*pinf)
    vinf = 1./np.sqrt(gamma) * (pinf - 1.) / np.sqrt(0.5*((gamma+1.)/gamma) * pinf+0.5*gamma1/gamma)
    einf = 0.5*rinf*vinf**2 + pinf/gamma1
    
    X, Y = grid.p_centers

    r = np.sqrt((X-x0)**2 + (Y-y0)**2)

    #First set the values for the cells that don't intersect the bubble boundary
    state.q[0,:,:] = rinf*(X<xshock) + rhoin*(r<=r0) + rhoout*(r>r0)*(X>=xshock)
    state.q[1,:,:] = rinf*vinf*(X<xshock)
    state.q[2,:,:] = 0.
    state.q[3,:,:] = einf*(X<xshock) + (pin*(r<=r0) + pout*(r>r0)*(X>=xshock))/gamma1
    state.q[4,:,:] = 1.*(r<=r0)

    #Now compute average density for the cells on the edge of the bubble
    d2 = np.linalg.norm(state.grid.delta)/2.
    dx = state.grid.delta[0]
    dy = state.grid.delta[1]
    dx2 = state.grid.delta[0]/2.
    dy2 = state.grid.delta[1]/2.
    for i in range(state.q.shape[1]):
        for j in range(state.q.shape[2]):
            ydown = Y[i,j]-dy2
            yup   = Y[i,j]+dy2
            if abs(r[i,j]-r0)<d2:
                infrac,abserr = integrate.quad(ycirc,X[i,j]-dx2,X[i,j]+dx2,args=(ydown,yup),epsabs=1.e-8,epsrel=1.e-5)
                infrac=infrac/(dx*dy)
                state.q[0,i,j] = rhoin*infrac + rhoout*(1.-infrac)
                state.q[3,i,j] = (pin*infrac + pout*(1.-infrac))/gamma1
                state.q[4,i,j] = 1.*infrac


def auxinit(state):
    """
    aux[1,i,j] = radial coordinate of cell centers for cylindrical source terms
    """
    y = state.grid.y.centers
    for j,r in enumerate(y):
        state.aux[0,:,j] = r


def incoming_shock(state,dim,t,qbc,auxbc,num_ghost):
    """
    Incoming shock at left boundary.
    """
    gamma1 = gamma - 1.

    pinf=5.
    rinf = (gamma1 + pinf*(gamma+1.))/ ((gamma+1.) + gamma1*pinf)
    vinf = 1./np.sqrt(gamma) * (pinf - 1.) / np.sqrt(0.5*((gamma+1.)/gamma) * pinf+0.5*gamma1/gamma)
    einf = 0.5*rinf*vinf**2 + pinf/gamma1

    for i in range(num_ghost):
        qbc[0,i,...] = rinf
        qbc[1,i,...] = rinf*vinf
        qbc[2,i,...] = 0.
        qbc[3,i,...] = einf
        qbc[4,i,...] = 0.


def step_Euler_radial(solver,state,dt):
    """
    Geometric source terms for Euler equations with cylindrical symmetry.
    Integrated using a 2-stage, 2nd-order Runge-Kutta method.
    This is a Clawpack-style source term routine, which approximates
    the integral of the source terms over a step.
    """
    dt2 = dt/2.

    q = state.q
    rad = state.aux[0,:,:]

    rho = q[0,:,:]
    u   = q[1,:,:]/rho
    v   = q[2,:,:]/rho
    press  = (gamma - 1.) * (q[3,:,:] - 0.5*rho*(u**2 + v**2))

    qstar = np.empty(q.shape)

    qstar[0,:,:] = q[0,:,:] - dt2/rad * q[2,:,:]
    qstar[1,:,:] = q[1,:,:] - dt2/rad * rho*u*v
    qstar[2,:,:] = q[2,:,:] - dt2/rad * rho*v*v
    qstar[3,:,:] = q[3,:,:] - dt2/rad * v * (q[3,:,:] + press)

    rho = qstar[0,:,:]
    u   = qstar[1,:,:]/rho
    v   = qstar[2,:,:]/rho
    press  = (gamma - 1.) * (qstar[3,:,:] - 0.5*rho*(u**2 + v**2))

    q[0,:,:] = q[0,:,:] - dt/rad * qstar[2,:,:]
    q[1,:,:] = q[1,:,:] - dt/rad * rho*u*v
    q[2,:,:] = q[2,:,:] - dt/rad * rho*v*v
    q[3,:,:] = q[3,:,:] - dt/rad * v * (qstar[3,:,:] + press)


def dq_Euler_radial(solver,state,dt):
    """
    Geometric source terms for Euler equations with radial symmetry.
    This is a SharpClaw-style source term routine, which returns
    the value of the source terms.
    """
    q   = state.q
    rad = state.aux[0,:,:]

    rho = q[0,:,:]
    u   = q[1,:,:]/rho
    v   = q[2,:,:]/rho
    press  = (gamma - 1.) * (q[3,:,:] - 0.5*rho*(u**2 + v**2))

    dq = np.empty(q.shape)

    dq[0,:,:] = -dt/rad * q[2,:,:]
    dq[1,:,:] = -dt/rad * rho*u*v
    dq[2,:,:] = -dt/rad * rho*v*v
    dq[3,:,:] = -dt/rad * v * (q[3,:,:] + press)
    dq[4,:,:] = 0

    return dq

def setup(use_petsc=False,solver_type='classic', outdir='_output', kernel_language='Fortran',
        disable_output=False, mx=320, my=80, tfinal=0.6, num_output_times = 10):
    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if solver_type=='sharpclaw':
        solver = pyclaw.SharpClawSolver2D(riemann.euler_5wave_2D)
        solver.dq_src = dq_Euler_radial
        solver.weno_order = 5
        solver.lim_type   = 2
    else:
        solver = pyclaw.ClawSolver2D(riemann.euler_5wave_2D)
        solver.step_source = step_Euler_radial
        solver.source_split = 1
        solver.limiters = [4,4,4,4,2]
        solver.cfl_max = 0.5
        solver.cfl_desired = 0.45

    x = pyclaw.Dimension(0.0,2.0,mx,name='x')
    y = pyclaw.Dimension(0.0,0.5,my,name='y')
    domain = pyclaw.Domain([x,y])

    num_aux=1
    state = pyclaw.State(domain,num_eqn,num_aux)
    state.problem_data['gamma']= gamma

    qinit(state)
    auxinit(state)

    solver.user_bc_lower = incoming_shock

    solver.bc_lower[0]=pyclaw.BC.custom
    solver.bc_upper[0]=pyclaw.BC.extrap
    solver.bc_lower[1]=pyclaw.BC.wall
    solver.bc_upper[1]=pyclaw.BC.extrap
    #Aux variable in ghost cells doesn't matter
    solver.aux_bc_lower[0]=pyclaw.BC.extrap
    solver.aux_bc_upper[0]=pyclaw.BC.extrap
    solver.aux_bc_lower[1]=pyclaw.BC.extrap
    solver.aux_bc_upper[1]=pyclaw.BC.extrap

    claw = pyclaw.Controller()
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver

    claw.keep_copy = True
    if disable_output or outdir is None:
        claw.output_format = None
    claw.tfinal = tfinal
    claw.num_output_times = num_output_times
    claw.outdir = outdir
    claw.setplot = setplot

    return claw

    
def setplot(plotdata):
    """ 
    Plot solution using VisClaw.
    """ 
    from clawpack.visclaw import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data
    
    # Pressure plot
    plotfigure = plotdata.new_plotfigure(name='Density', figno=0)

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Density'
    plotaxes.scaled = True      # so aspect ratio is 1
    plotaxes.afteraxes = label_axes

    plotitem = plotaxes.new_plotitem(plot_type='2d_schlieren')
    plotitem.plot_var = 0
    plotitem.add_colorbar = False
    

    # Tracer plot
    plotfigure = plotdata.new_plotfigure(name='Tracer', figno=1)

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Tracer'
    plotaxes.scaled = True      # so aspect ratio is 1
    plotaxes.afteraxes = label_axes

    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.pcolor_cmin = 0.
    plotitem.pcolor_cmax=1.0
    plotitem.plot_var = 4
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = False
    

    # Energy plot
    plotfigure = plotdata.new_plotfigure(name='Energy', figno=2)

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Energy'
    plotaxes.scaled = True      # so aspect ratio is 1
    plotaxes.afteraxes = label_axes

    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.pcolor_cmin = 2.
    plotitem.pcolor_cmax=18.0
    plotitem.plot_var = 3
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = False
    
    return plotdata

def label_axes(current_data):
    import matplotlib.pyplot as plt
    plt.xlabel('z')
    plt.ylabel('r')
    

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



#!/usr/bin/env python

import os
import sys
import glob
import datetime

import numpy

# Plot customization
import matplotlib

# Markers and line widths
matplotlib.rcParams['lines.linewidth'] = 2.0
matplotlib.rcParams['lines.markersize'] = 6
matplotlib.rcParams['lines.markersize'] = 8

# Font Sizes
matplotlib.rcParams['font.size'] = 16
matplotlib.rcParams['axes.labelsize'] = 16
matplotlib.rcParams['legend.fontsize'] = 12
matplotlib.rcParams['xtick.labelsize'] = 16
matplotlib.rcParams['ytick.labelsize'] = 16

# DPI of output images
matplotlib.rcParams['savefig.dpi'] = 300

import matplotlib.pyplot as plt

convert2rgbfloat = lambda rgb: [value / 256.0 for value in rgb]
days2seconds = lambda days: days * 60.0**2 * 24.0
seconds2days = lambda seconds: seconds / (60.0**2 * 24.0)

def set_day_ticks(new_ticks=[-3, -2, -1, 0]):
    plt.xticks(new_ticks, [str(tick) for tick in new_ticks])

def set_cell_ticks():
    # plt.ticklabel_format(style='sci')
    locs,labels = plt.yticks()
    labels = locs / 1e6
    plt.yticks(locs,labels)
    # plt.yticks(new_ticks, [str(tick) for tick in new_ticks])

if __name__ == "__main__":

    output_path = "./_output"    
    if len(sys.argv) > 1:
        output_path = sys.argv[1]

    num_levels = 7
    # ADCIRC_num_nodes = 3331560
    landfall = datetime.datetime(2008,9,13 - 1,7) - datetime.datetime(2008,1,1,0)
    landfall = days2seconds(landfall.days) + landfall.seconds

    file_list = glob.glob(os.path.join(output_path,"fort.q*"))

    time = numpy.empty(len(file_list), dtype=float)
    num_grids = numpy.zeros((time.shape[0], num_levels), dtype=int)
    num_cells = numpy.zeros((time.shape[0], num_levels), dtype=int)

    for (n,path) in enumerate(file_list):
        # Read t file
        t_path = path[:-5] + "t" + path[-4:]
        t_file = open(t_path, 'r')
        time[n] = seconds2days(float(t_file.readline().split()[0]) - landfall)
        t_file.readline()
        t_file_num_grids = int(t_file.readline().split()[0])
        t_file.close()

        # Read q_file
        q_file = open(path, 'r')
        line = "\n"
        while line != "":
            line = q_file.readline()
            if "grid_number" in line:
                # print "grid number:", int(line.split()[0])
                level = int(q_file.readline().split()[0])
                num_grids[n,level - 1] += 1 
                mx = int(q_file.readline().split()[0])
                my = int(q_file.readline().split()[0])
                num_cells[n,level - 1] += mx * my

        q_file.close()

        # File checking
        if numpy.sum(num_grids[n,:]) != t_file_num_grids:
            raise Exception("Number of grids in fort.t* file and fort.q* file do not match.")

    # Plot cascading time histories per level
    colors = [ (value / 256.0, value / 256.0, value / 256.0) 
                                    for value in [247, 217, 189, 150, 115, 82, 37] ]
    proxy_artists = [plt.Rectangle((0, 0), 1, 1, fc=colors[level], 
            label="Level %s" % (str(level+1))) for level in range(num_levels)]

    # Number of grids
    fig = plt.figure()
    axes = fig.add_subplot(111)
    axes.set_yscale('log')
    axes.stackplot(time ,num_grids.transpose(), colors=colors)
    axes.set_xlabel('Days from landfall')
    plt.subplots_adjust(left=0.13, bottom=0.12, right=0.90, top=0.90)
    axes.set_xlim([-2,0.75])
    axes.set_ylim([0,1e4])
    set_day_ticks()
    axes.set_ylabel('Number of Grids')
    axes.set_title("Number of Grids per Level in Time")
    axes.legend(proxy_artists, ["Level %s" % (str(level+1)) for level in range(num_levels)], loc=2)
    fig.savefig("num_grids.png")

    # Number of cells
    fig = plt.figure()
    axes = fig.add_subplot(111)
    axes.set_yscale('log')
    axes.stackplot(time, num_cells.transpose(), colors=colors)
    axes.set_xlim([-2,0.75])
    axes.set_ylim([0,1e7])
    set_day_ticks()
    plt.subplots_adjust(left=0.13, bottom=0.12, right=0.90, top=0.90)
    axes.set_xlabel('Days from landfall')
    axes.set_ylabel('Number of Cells')
    axes.set_title("Number of Cells per Level in Time")
    axes.legend(proxy_artists, ["Level %s" % (str(level+1)) for level in range(num_levels)], loc=2)
    fig.savefig("num_cells.png")


    plt.show()





#!/usr/bin/env python
# Permute indices on Fortran arrays

# Written by @gradylemoine

import re, sys

def permute_file(infile, outfile, permute_list, write_header=True):
    '''permute_file(INFILE, OUTFILE, PERMUTE_LIST):
Permute array indices in a Fortran source file.  INFILE is the name
of the inputfile, and OUTFILE is the name of the output file.  It's
an error if INFILE == OUTFILE.  See permute_indices for documentation
on PERMUTE_LIST.'''

    if (infile == outfile):
        print('Identical filenames specified for input and output; not doing anything')
        return

    f = open(infile, 'r')    # No error checking
    code_in = f.read()
    f.close()

    err_list, code_out = permute_indices(code_in, permute_list)

    f = open(outfile, 'w')    # No error checking
    if (write_header):
        f.write("! *** DO NOT EDIT THIS FILE ***\n")
        f.write("! This file was generated automatically from %s using\n"%infile)
        f.write("! the permute.py Python script to permute the array\n")
        f.write("! indices of the following variables:\n")
        f.write("! %s\n"%(', '.join([str(x[0]) for x in permute_list])))
        f.write("! If this file is wrong, edit the parent file (for\n")
        f.write("! algorithmic errors) or the permute.py script\n")
        f.write("! (for array permutation errors)\n\n")
    f.write(code_out)
    f.close()
    for lineno, bad_vars, line in err_list:
        print(('Line %4d: confused about '%lineno)+', '.join(bad_vars)+' in: '+line)
    

def permute_indices(code, permute_list):
    '''permute_indices(CODE, PERMUTE_LIST):
Permute array indices on a single line.  CODE is the Fortran code to
be operated on; PERMUTE_LIST is a list of tuples (NAME, PLIST), where
NAME is the name of an array to be permuted, and PLIST is a list of
integers saying which indices of the original array go where in the
final list.  (This list is indexed starting from 1, because it's a
list of regex backrefences.)

Returns a tuple (ERR_LIST, CODE_OUT), where ERR_LIST is a list tuples
of the lines that weren't successfully processed, the variables that
weren't successfully processed on them, and the original line, and
CODE_OUT is the modified code.'''

    regex_match = []    # To be used with re.findall()
    regex_replace = []  # To be used with re.subn()
    str_replace = []    # String to replace with in re.subn()
    for name, plist in permute_list:
        matchstr = r'\b'+name+r'\b'
        repstr = (r'\b'+name+r'\s*\('
                  +r','.join([r'\s*([^,\)]+?)\s*']*len(plist))
                  +r'\)')
        regex_match.append(re.compile(matchstr, re.IGNORECASE))
        regex_replace.append(re.compile(repstr, re.IGNORECASE))
        str_replace.append(name+'('+','.join([r'\%d'%n for n in plist])+')')

    iscomment = re.compile('^(c|\s*!)', re.IGNORECASE)

    lines = code.split('\n')
    code_out = ''
    err_list = []
    for i in range(len(lines)):
        bad_vars = []
        line = lines[i]
        #if (not iscomment.match(line)):
        if True:
            
            line0 = line
            for j in range(len(permute_list)):
                nmatch = len(regex_match[j].findall(line))
                line_new, nrep = regex_replace[j].subn(str_replace[j], line)
                if (nrep == nmatch):
                    line = line_new
                else:
                    bad_vars.append(permute_list[j][0])

            if (len(bad_vars) > 0):
                err_list.append((i+1, bad_vars, line0))

        code_out = code_out + line + '\n'

    return (err_list, code_out)


if (__name__ == '__main__'):
    if (len(sys.argv) < 3):
        print('permute.py: need source and destination file specified on command line')
        
    else:
        # assuming 2d:
        plist = [('q',     [3,1,2]),
                 ('aux',   [3,1,2])]

        permute_file(sys.argv[1], sys.argv[2], plist, write_header=False)



from . import acoustics_2d_interface
import numpy as np
from clawpack.pyclaw.util import check_diff
import os

def check_error(**kwargs):

    solver_type = kwargs['solver_type']
    claw = acoustics_2d_interface.setup(disable_output=True,**kwargs)
    claw.run()
    test_pressure = claw.frames[-1].q[0,:,:]

    thisdir = os.path.dirname(__file__)
    expected_pressure = np.loadtxt(os.path.join(thisdir, 'pressure_%s.txt' % solver_type))
    test_err = np.max(np.abs(expected_pressure[:].reshape(-1) - 
                             test_pressure[:].reshape(-1)))
    print(test_err)
    return check_diff(0, test_err, abstol=1e-1)


class TestAcoustics2D:
    def test_classic(self):
        assert check_error(solver_type='classic',num_cells=(50,50))==None

    def test_sharpclaw(self):
        assert check_error(solver_type='sharpclaw',num_cells=(50,50))==None

    def test_sharpclaw_multistep(self):
        assert check_error(solver_type='sharpclaw',num_cells=(50,50),
                           time_integrator='SSPLMMk2')==None




from pylab import *
import os
from clawpack.visclaw import plottools, geoplot
from clawpack.visclaw import animation_tools
from matplotlib import animation, colors
from clawpack.geoclaw import fgout_tools
from datetime import timedelta 

fgno = 1
outdir = '_output'
output_format = 'binary'

# Instantiate object for reading fgout frames:
fgout_grid = fgout_tools.FGoutGrid(fgno, outdir, output_format)

# Plot one frame of fgout data 
fgframe = 20
fgout = fgout_grid.read_frame(2)

fgout = fgout_grid.read_frame(fgframe)

figure(1, figsize=(8,8))
imshow(flipud(fgout.B.T), extent=fgout.extent_edges,
       cmap=geoplot.land_colors)

clim(0,100)

eta_water = where(fgout.h > 0, fgout.eta, nan)
imshow(flipud(eta_water.T), extent=fgout.extent_edges,
       cmap=geoplot.tsunami_colormap)

clim(-0.2, 0.2)

title('Surface at time %s' % timedelta(seconds=fgout.t))

fname = 'fgout_frame%s.png' % str(fgframe).zfill(4)
savefig(fname)
print('Created ',fname)




#!/usr/bin/env python
# encoding: utf-8
r"""
F-wave Riemann solver for nonlinear elasticity in heterogeneous media

.. math:: q_t + f(q,x)_x = 0

where

.. math::
    q(x,t) = \left [ \begin{array}{c} \epsilon(x,t) \\ \rho(x) u(x,t) \end{array} \right ]

and the flux vector is

.. math::

    f(q,x) = \left [ \begin{array}{c} -u \\ \sigma(\epsilon,x) \end{array} \right ]

Two stress-strain relations are included.  The default is

..  math:: \sigma(\epsilon,K) = \exp(K(x) \epsilon) - 1

By setting problem_data['stress_relation'] = 'quadratic' one can use instead

..  math:: \sigma(\epsilon,K_1,K_2) = K_1 \epsilon + K_2 \epsilon^2

For the latter relation, K_2 is stored in a third aux entry (aux[2,:]).
"""

import numpy as np

strain = 0
momentum = 1
density = 0
K = 1
K1 = 1
K2 = 2


def nonlinear_elasticity_1D(q_l, q_r, aux_l, aux_r, problem_data):
    r"""
    1d nonlinear elasticity riemann solver

    *aux* is expected to contain -
     - aux[0,i] - density in cell i
     - aux[1,i] - bulk modulus in cell i
     - aux[2,i] - K2 (needed for quadratic stress relation only)

    See :ref:`pyclaw_rp` for more details.
    """

    num_eqn = 2
    num_waves = 2
    # Convenience
    num_rp = np.size(q_l, 1)

    # Set up arrays for return values
    fwave = np.empty( (num_eqn, num_waves, num_rp) )
    s = np.empty( (num_waves, num_rp) )
    amdq = np.empty( (num_eqn, num_rp) )
    apdq = np.empty( (num_eqn, num_rp) )

    sigma_l = sigma(q_l, aux_l, problem_data)
    sigma_r = sigma(q_r, aux_r, problem_data)
    bulk_l = sigmap(q_l, aux_l, problem_data)
    bulk_r = sigmap(q_r, aux_r, problem_data)

    # Linearized bulk modulus, sound speed, and impedance:
    cl = np.sqrt(bulk_l/aux_l[density,:])
    cr = np.sqrt(bulk_r/aux_r[density,:])
    zl = cl*aux_l[density,:]
    zr = cr*aux_r[density,:]

    # Jumps:
    du = q_r[momentum,:]/aux_r[density,:]-q_l[momentum,:]/aux_l[density,:]
    dsig = sigma_r - sigma_l

    b1 = - (zr*du + dsig) / (zr+zl)
    b2 = - (zl*du - dsig) / (zr+zl)

    # Compute the f-waves
    # 1-Wave
    fwave[0,0,:] = b1
    fwave[1,0,:] = b1 * zl
    s[0,:] = -cl

    # 2-Wave
    fwave[0,1,:] = b2
    fwave[1,1,:] = b2 * (-zr)
    s[1,:] = cr

    # Compute the left going and right going fluctuations
    for m in range(num_eqn):
        amdq[m,:] = fwave[m,0,:]
        apdq[m,:] = fwave[m,1,:]

    return fwave, s, amdq, apdq


def sigma(q, aux, problem_data):
    r"""Stress-strain relation."""
    stress_relation = problem_data.get('stress_relation', 'exponential')
    if stress_relation == 'exponential':
        return np.exp(aux[K,:]*q[strain,:]) - 1.
    elif stress_relation == 'quadratic':
        return aux[K1,:]*q[strain,:] + aux[K2,:]*q[strain,:]**2


def sigmap(q, aux, problem_data):
    r"""Derivative of stress-strain relation w.r.t. strain."""
    stress_relation = problem_data.get('stress_relation', 'exponential')
    if stress_relation == 'exponential':
        return aux[K,:]*np.exp(aux[K,:]*q[strain,:])
    elif stress_relation == 'quadratic':
        return aux[K1,:] + 2*aux[K2,:]*q[strain,:]



#!/usr/bin/env python
# encoding: utf-8
r"""
Shock-tube problem
===================================

Solve the one-dimensional Euler equations for inviscid, compressible flow:

.. math::
    \rho_t + (\rho u)_x & = 0 \\
    (\rho u)_t + (\rho u^2 + p)_x & = 0 \\
    E_t + (u (E + p) )_x & = 0.

The fluid is an ideal gas, with pressure given by :math:`p=\rho (\gamma-1)e` where
e is internal energy.

This script runs a shock-tube problem.
"""
from clawpack import riemann
from clawpack.riemann.euler_with_efix_1D_constants import density, momentum, energy, num_eqn

gamma = 1.4 # Ratio of specific heats

def setup(use_petsc=False, outdir='./_output', solver_type='classic',
          kernel_language='Python',disable_output=False):

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if kernel_language =='Python':
        rs = riemann.euler_1D_py.euler_hllc_1D
    elif kernel_language =='Fortran':
        rs = riemann.euler_hlle_1D

    if solver_type=='sharpclaw':
        solver = pyclaw.SharpClawSolver1D(rs)
    elif solver_type=='classic':
        solver = pyclaw.ClawSolver1D(rs)

    solver.kernel_language = kernel_language

    solver.bc_lower[0]=pyclaw.BC.extrap
    solver.bc_upper[0]=pyclaw.BC.extrap

    mx = 800
    x = pyclaw.Dimension(-1.0,1.0,mx,name='x')
    domain = pyclaw.Domain([x])
    state = pyclaw.State(domain,num_eqn)

    state.problem_data['gamma'] = gamma
    state.problem_data['gamma1'] = gamma - 1.

    x = state.grid.x.centers

    rho_l = 1.; rho_r = 1./8
    p_l = 1.; p_r = 0.1
    state.q[density ,:] = (x<0.)*rho_l + (x>=0.)*rho_r
    state.q[momentum,:] = 0.
    velocity = state.q[momentum,:]/state.q[density,:]
    pressure = (x<0.)*p_l + (x>=0.)*p_r
    state.q[energy  ,:] = pressure/(gamma - 1.) + 0.5 * state.q[density,:] * velocity**2

    claw = pyclaw.Controller()
    claw.tfinal = 0.4
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    claw.num_output_times = 10
    claw.outdir = outdir
    claw.setplot = setplot
    claw.keep_copy = True
    if disable_output:
        claw.output_format = None

    return claw

#--------------------------
def setplot(plotdata):
#--------------------------
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    """ 
    plotdata.clearfigures()  # clear any old figures,axes,items data

    plotfigure = plotdata.new_plotfigure(name='', figno=0)

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(211)'
    plotaxes.title = 'Density'

    plotitem = plotaxes.new_plotitem(plot_type='1d')
    plotitem.plot_var = density
    plotitem.kwargs = {'linewidth':3}
    
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(212)'
    plotaxes.title = 'Energy'

    plotitem = plotaxes.new_plotitem(plot_type='1d')
    plotitem.plot_var = energy
    plotitem.kwargs = {'linewidth':3}
    
    return plotdata

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



from __future__ import absolute_import
from six.moves import range
def interactive_plot(outdir='./_output',file_format='petsc',setplot=None):
    """
    Convenience function for launching an interactive plotting session.
    """
    from clawpack.pyclaw.plot import plot
    plot(setplot,outdir=outdir,file_format=file_format,iplot=True,htmlplot=False)

def html_plot(outdir='./_output',file_format='petsc',setplot=None):
    """
    Convenience function for creating html page with plots.
    """
    from clawpack.pyclaw.plot import plot
    plot(setplot,outdir=outdir,file_format=file_format,htmlplot=True,iplot=False)

def plotPetsc(clawobj,delay=1):
    """
    Takes either a controller or solution object and prints each frame
    using PETSc.Viewer.
    """
    from petsc4py import PETSc
    from clawpack.pyclaw import controller, solution

    if isinstance(clawobj,controller.Controller):
        for n in range(0,clawobj.num_output_times):
            sol = clawobj.frames[n]
            viewer = PETSc.Viewer.DRAW(sol.patch.gqVec.comm)
            OptDB = PETSc.Options()
            OptDB['draw_pause'] = delay
            viewer(sol.patch.gqVec)

    elif isinstance(clawobj,solution.Solution):
        viewer = PETSc.Viewer.DRAW(clawobj.patch.gqVec.comm)
        OptDB = PETSc.Options()
        OptDB['draw_pause'] = -1
        viewer(clawobj.patch.gqVec)






""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

from __future__ import absolute_import
from __future__ import print_function

#--------------------------
def setplot(plotdata=None):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of clawpack.visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 


    if plotdata is None:
        from clawpack.visclaw.data import ClawPlotData
        plotdata = ClawPlotData()

    plotdata.clearfigures()  # clear any old figures,axes,items data

    def draw_interface_add_legend(current_data):
        from pylab import plot
        plot([0., 0.], [-1000., 1000.], 'k--')
        try:
            from clawpack.visclaw import legend_tools
            labels = ['Level 1','Level 2', 'Level 3']
            legend_tools.add_legend(labels, colors=['g','b','r'],
                        markers=['^','s','o'], linestyles=['','',''],
                        loc='upper left')
        except:
            pass

    def draw_interface_add_legend_innerprod(current_data):
        from pylab import plot
        plot([0., 0.], [-1000., 1000.], 'k--')
        try:
            from clawpack.visclaw import legend_tools
            labels = ['Level 1','Level 2']
            legend_tools.add_legend(labels, colors=['g','b'],
                                    markers=['^','s'], linestyles=['',''],
                                    loc='upper left')
        except:
            pass


    # Figure for q[0]
    plotfigure = plotdata.new_plotfigure(name='Pressure and Velocity', figno=1)
    plotfigure.kwargs = {'figsize': (8,8)}
    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(2,1,1)'   # top figure
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = [-.5,1.1]
    plotaxes.title = 'Pressure'
    plotaxes.afteraxes = draw_interface_add_legend

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 0
    plotitem.amr_color = ['g','b','r']
    plotitem.amr_plotstyle = ['^-','s-','o-']
    plotitem.amr_data_show = [1,1,1]
    plotitem.amr_kwargs = [{'markersize':5},{'markersize':4},{'markersize':3}]

    # Figure for q[1]

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(2,1,2)'   # bottom figure
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = [-.5,1.1]
    plotaxes.title = 'Velocity'
    plotaxes.afteraxes = draw_interface_add_legend

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 1
    plotitem.amr_color = ['g','b','r']
    plotitem.amr_plotstyle = ['^-','s-','o-']
    plotitem.amr_data_show = [1,1,1]
    plotitem.amr_kwargs = [{'markersize':5},{'markersize':4},{'markersize':3}]
    
    # Figure for inner product, q[2]
    
    plotfigure = plotdata.new_plotfigure(name='Inner Product', figno=10)
    
    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = [-0.1,1]         # use when taking inner product with forward solution
    #plotaxes.ylimits = [-0.01,0.02]    # use when taking inner product with Richardson error
    plotaxes.title = 'Inner Product'
    plotaxes.afteraxes = draw_interface_add_legend_innerprod
    
    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d')
    plotitem.plot_var = plot_innerprod
    plotitem.amr_color = ['g','b','r']
    plotitem.amr_plotstyle = ['^-','s-','o-']
    plotitem.amr_data_show = [1,1,0]
    plotitem.show = True       # show on plot?

    
    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='q', figno=300, \
                                         type='each_gauge')
    plotfigure.clf_each_gauge = True
    plotfigure.kwargs = {'figsize': (10,10)}
                                         
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(211)'
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Pressure'
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 0
    plotitem.plotstyle = 'b-'

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(212)'
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Velocity'
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 1
    plotitem.plotstyle = 'b-'

    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via clawpack.visclaw.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata

def plot_innerprod(current_data):
    return current_data.aux[2,:]
    



"""
Make an mp4 animation of fgout grid results that also includes a transect
of the solution, as often desired in animations.

This is done in a way that makes the animation quickly and with minimum 
storage required, by making one plot and then defining an update function
that only changes the parts of the plot that change in each frame.

Make the animation via:
    python make_fgout_animation_transect.py

If this script is executed in IPython or a notebook it may go into
an infinite loop for reasons unknown.  If so, close the figure to halt.

To view individual fgout frames interactively, this should work:
    import make_fgout_animation_transect
    make_fgout_animation_transect.update(fgframeno)  # for desired fgout frameno

Uses blit==False so that update_artists tuple does not need to be returned
from the update function.
"""

import sys
if 'matplotlib' not in sys.modules:
    # Use an image backend to insure animation has size specified by figsize
    import matplotlib
    matplotlib.use('Agg')

from pylab import *
import os, glob
from clawpack.visclaw import plottools, geoplot, gridtools, animation_tools
from clawpack.geoclaw import util
from matplotlib import animation, colors
from datetime import timedelta

from clawpack.geoclaw import fgout_tools
    
fgno = 1  # which fgout grid

outdir = '_output'

if 1:
    # use all fgout frames in outdir:
    fgout_frames = glob.glob(os.path.join(outdir, \
                                          'fgout%s.t*' % str(fgno).zfill(4)))
    nout = len(fgout_frames)
    fgframes = range(1, nout+1)
    print('Found %i fgout frames in %s' % (nout,outdir))
else:
    # set explicitly, e.g. to test with only a few frames
    fgframes = range(1,26)  # frames of fgout solution to use in animation

# Instantiate object for reading fgout frames:
fgout_grid = fgout_tools.FGoutGrid(fgno, outdir) 
fgout_grid.read_fgout_grids_data()


# Plot one frame of fgout data and define the Artists that will need to
# be updated in subsequent frames:

fgout = fgout_grid.read_frame(fgframes[0])

plot_extent = fgout.extent_edges
ylat = fgout.Y.mean()  # for aspect ratio of plots

fig = figure(figsize=(12,7))

# ---------------------------------
# axis for planview plot of ocean:
ax = axes([.1,.1,.4,.8])

ax.set_xlim(plot_extent[:2])
ax.set_ylim(plot_extent[2:])

plottools.pcolorcells(fgout.X,fgout.Y,fgout.B, cmap=geoplot.land_colors)
clim(0,100)

eta = ma.masked_where(fgout.h<0.001, fgout.eta)

eta_plot = plottools.pcolorcells(fgout.X,fgout.Y,eta,
                                 cmap=geoplot.tsunami_colormap)
clim(-0.3,0.3)
cb = colorbar(eta_plot, extend='both', shrink=0.5,
              orientation='horizontal', anchor=(0.4,1))
cb.set_label('meters')
title_text = title('Surface at time %s' % timedelta(seconds=fgout.t))

ax.set_aspect(1./cos(ylat*pi/180.))
ticklabel_format(useOffset=False)
xticks(rotation=20)
ax.set_xlim(plot_extent[:2])
ax.set_ylim(plot_extent[2:])

# ---------------------------------
# Transect:

x1trans = -110.; y1trans = -20.
x2trans = -72.;  y2trans = -35.

plot([x1trans,x2trans], [y1trans,y2trans],'k-',linewidth=0.8)
text(x1trans+1,y1trans+1,'Transect', ha='center', fontsize=10)

# define points on transect:
npts = 1000  # number of points on transect
if 0:
    # straight line on longitude-latitude plane:
    xtrans = linspace(x1trans,x2trans,npts)
    ytrans = linspace(y1trans,y2trans,npts)
else:
    # great circle on earth:
    xtrans,ytrans = util.gctransect(x1trans,y1trans,x2trans,y2trans,npts)
    

def extract_transect(fgout_soln,xtrans,ytrans):
    """
    Interpolate from fgout_solution to points on the transect, taking value
    from cell the fgout point lies in, giving piecewise constant interpolant
    when npts is large.
    Alternatively could use method='linear' for linear interpolation.
    """

    eta1d = gridtools.grid_eval_2d(fgout_soln.X, fgout_soln.Y,
                                   fgout_soln.eta, xtrans, ytrans,
                                   method='nearest')
    B1d = gridtools.grid_eval_2d(fgout_soln.X, fgout_soln.Y,
                                 fgout_soln.B, xtrans, ytrans,
                                 method='nearest')
    eta1d = where(B1d>0, nan, eta1d)   # mask onshore region
    return B1d, eta1d

# ----------------------------------------
# Axes for transect of surface elevation:

axtrans1 = axes([.55,.6,.4,.3])

axtrans1.set_title('Surface on transect')

axtrans1.set_xlim(x1trans,x2trans)
axtrans1.set_ylim(-1,1)

Btrans1, etatrans1 = extract_transect(fgout,xtrans,ytrans)
#import pdb; pdb.set_trace()

Btrans, etatrans = Btrans1, etatrans1

# surface plot:
etatrans1_plot, = axtrans1.plot(xtrans, etatrans, 'b')
axtrans1.grid(True)

# ----------------------------------------
# Axes for transect of topography:

axtrans2 = axes([.55,.1,.4,.3])

axtrans2.set_title('Topography on transect')

axtrans2.set_xlim(x1trans,x2trans)
axtrans2.set_ylim(-5000,1000)

Btrans, etatrans = extract_transect(fgout,xtrans,ytrans)

# filled regions:
Bfill_plot = axtrans2.fill_between(xtrans, Btrans-1e5, Btrans, 
                                   color=[.7,1,.7,1]) # light green
etafill_plot = axtrans2.fill_between(xtrans, Btrans, etatrans, 
                                     color=[.7,.7,1,1])  # light blue

# surface and topo solid lines:
etatrans2_plot, = axtrans2.plot(xtrans, etatrans, 'b')
Btrans2_plot, = axtrans2.plot(xtrans, Btrans, 'g')


# create a dummy figure and axes, only needed to update fill_between artists:
figdummy,axdummy = subplots()


def update(fgframeno):
    """
    Update an exisiting plot with solution from fgout frame fgframeno.
    Assumes blit==False in call to animation.FuncAnimation below,
    so tuple of update_artists does not need to be returned.
    """
    
    fgout = fgout_grid.read_frame(fgframeno)
    print('Updating plot at time %s' % timedelta(seconds=fgout.t))
        
    # reset title to current time:
    title_text.set_text('Surface at time %s' % timedelta(seconds=fgout.t))

    # reset surface eta to current state:
    eta = ma.masked_where(fgout.h<0.001, fgout.eta)
    eta_plot.set_array(eta.T.flatten())
        
    # update transect data:
    Btrans, etatrans = extract_transect(fgout,xtrans,ytrans)    
    
    # update lines plotted:
    etatrans1_plot.set_data(xtrans,etatrans)
    etatrans2_plot.set_data(xtrans,etatrans)
    Btrans2_plot.set_data(xtrans,Btrans)

    # update the PolyCollections for fill_between plots:
    # There doesn't seem to be an easier way to do this...             
    dummy = axdummy.fill_between(xtrans, Btrans-1e5, Btrans, color=[.5,1,.5,1])
    dp = dummy.get_paths()[0]
    dummy.remove()
    Bfill_plot.set_paths([dp.vertices])

    dummy = axdummy.fill_between(xtrans, Btrans, etatrans, color=[.5,.5,1,1])
    dp = dummy.get_paths()[0]
    dummy.remove()
    etafill_plot.set_paths([dp.vertices])
    

if __name__ == '__main__':

    print('Making anim...')
    anim = animation.FuncAnimation(fig, update, frames=fgframes, 
                                   interval=200, blit=False)
    
    # Output files:
    name = 'fgout_animation_with_transect'

    fname_mp4 = name + '.mp4'

    #fname_html = None
    fname_html = name + '.html'

    if fname_mp4:
        fps = 5
        print('Making mp4...')
        writer = animation.writers['ffmpeg'](fps=fps)
        anim.save(fname_mp4, writer=writer)
        print("Created %s" % fname_mp4)

    if fname_html:
        # html version:
        animation_tools.make_html(anim, file_name=fname_html, title=name)




"""
Create fgmax_grid.txt input file
"""

from __future__ import absolute_import
from clawpack.geoclaw import fgmax_tools
import os


def make_fgmax_grid1(datadir):
    fg = fgmax_tools.FGmaxGrid()
    fg.point_style = 2       # will specify a 2d grid of points
    fg.x1 = -2.
    fg.x2 = 2.
    fg.y1 = -2.
    fg.y2 = 2.
    fg.dx = 0.1
    fg.tstart_max = 0.        # when to start monitoring max values
    fg.tend_max = 1.e10       # when to stop monitoring max values
    fg.dt_check = 0.1         # target time (sec) increment between updating
                               # max values
    fg.min_level_check = 2    # which levels to monitor max on
    fg.arrival_tol = 1.e-2    # tolerance for flagging arrival

    fg.input_file_name = os.path.join(datadir, 'fgmax1.txt')
    fg.write_input_data()


if __name__ == "__main__":
    make_fgmax_grid1('.')





#!/usr/bin/env python
# encoding: utf-8

""" 
Test problem demonstrating a Sedov blast wave problem.
A spherical step function energy perturbation is initialized at the center of
the domain.  This creates an expanding shock wave.

This problem evolves the 3D Euler equations.
The primary variables are: 
    density (rho), x,y, and z momentum (rho*u,rho*v,rho*w), and energy.
"""
import numpy as np
from scipy import integrate
from clawpack import riemann
from clawpack.riemann.euler_3D_constants import density, x_momentum, \
                y_momentum, z_momentum, energy, num_eqn

gamma = 1.4 # Ratio of Specific Heats

x0 = 0.0; y0 = 0.0; z0 = 0.0 # Sphere location
rmax = 0.10 # Radius of Sedov Sphere

def sphere_top(y, x):
    z2 = rmax**2 - (x-x0)**2 - (y-y0)**2
    if z2 < 0:
        return 0
    else:
        return np.sqrt(z2)

def sphere_bottom(y, x):
    return -sphere_top(y,x)

def f(y, x, zdown, zup):
    top = min(sphere_top(y,x), zup)
    bottom = min(top,max(sphere_bottom(y,x), zdown))
    return top-bottom


def setup(kernel_language='Fortran', solver_type='classic', use_petsc=False,
          dimensional_split=False, outdir='Sedov_output', output_format='hdf5',
          disable_output=False, num_cells=(64,64,64),
          tfinal=0.10, num_output_times=10):

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if solver_type=='classic':
        solver = pyclaw.ClawSolver3D(riemann.euler_3D)
        solver.dimensional_split = dimensional_split
        solver.limiters = pyclaw.limiters.tvd.minmod
        solver.cfl_max = 0.6
        solver.cfl_desired = 0.55
        solver.dt_initial = 3e-4
    else:
        raise Exception('Unrecognized solver_type.')

    x = pyclaw.Dimension(-1.0, 1.0, num_cells[0], name='x')
    y = pyclaw.Dimension(-1.0, 1.0, num_cells[1], name='y')
    z = pyclaw.Dimension(-1.0, 1.0, num_cells[2], name='z')
    domain = pyclaw.Domain([x,y,z])

    state = pyclaw.State(domain,num_eqn)

    state.problem_data['gamma']=gamma
    
    grid = state.grid
    X,Y,Z = grid.p_centers
    r = np.sqrt((X-x0)**2 + (Y-y0)**2 + (Z-z0)**2)

    state.q[density,   :,:,:] = 1.0
    state.q[x_momentum,:,:,:] = 0.
    state.q[y_momentum,:,:,:] = 0.
    state.q[z_momentum,:,:,:] = 0.
    
    background_pressure = 1.0e-2
    Eblast = 0.851072
    pressure_in = Eblast*(gamma-1.)/(4./3.*np.pi*rmax**3)
    state.q[energy,:,:,:] = background_pressure/(gamma-1.) # energy (e)

    # Compute cell fraction inside initial perturbed sphere
    dx, dy, dz = state.grid.delta
    dx2, dy2, dz2 = [d/2. for d in state.grid.delta]
    dmax = max(state.grid.delta)

    for i in range(state.q.shape[1]):
        for j in range(state.q.shape[2]):
            for k in range(state.q.shape[3]):
                if r[i,j,k] - dmax > rmax:
                    continue
                xdown = X[i,j,k] - dx2
                xup   = X[i,j,k] + dx2
                ydown = lambda x : Y[i,j,k] - dy2
                yup   = lambda x : Y[i,j,k] + dy2
                zdown = Z[i,j,k] - dz2
                zup   = Z[i,j,k] + dz2

                infrac,abserr = integrate.dblquad(f,xdown,xup,ydown,yup,args=(zdown,zup),epsabs=1.e-3,epsrel=1.e-2)
                infrac=infrac/(dx*dy*dz)

                p = background_pressure + pressure_in*infrac # pressure
                state.q[energy,i,j,k] = p/(gamma-1.) # energy (e)

    solver.all_bcs = pyclaw.BC.extrap

    claw = pyclaw.Controller()
    claw.solution = pyclaw.Solution(state, domain)
    claw.solver = solver
    claw.output_format = output_format
    claw.keep_copy = True
    if disable_output:
        claw.output_format = None
    claw.tfinal = tfinal
    claw.num_output_times = num_output_times
    claw.outdir = outdir

    return claw

# __main__()
if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup)



#!/usr/bin/env python
from __future__ import absolute_import
from __future__ import print_function
import numpy as np
from petsc4py import PETSc
from six.moves import range

class PetCLAW:
    def advection1D(self, M, cfl, T):
        '''Script to solve 1D advection equation:
        q_t + q_x = 0
        Using first-order finite differences'''
        
        # PETSc DA object, which handles structured grids, here are
        # requesting M global points 
        da = PETSc.DA().create([M])
        print(da.getSizes())

        ranges = da.getRanges()
        print(ranges[0][1] - ranges[0][0])
        

        # this solves the problem on the domain [0 1] by default
        da.setUniformCoordinates() 

        # view commands dump to the screen
        da.view()

        # this gets the coordinate vector, this is akin to the
        # linspace(0,1,M) command 
        xvec = da.getCoordinates()
        xvec.view()
        # access the local data array within the globally distributed
        # vector 
        x = xvec.getArray()

        h = x[1]-x[0]
        k=cfl*h

        # global vector represents coordinated data with other
        # processors 
        fg = da.createGlobalVector()
        # local vector represents local data that is not shared
        fl = da.createLocalVector()

        # we will operate on the local vector, then coordinate with
        # other processors on the global vector 

        N=int(round(T/k))
        
        # Initial condition:
        q = np.exp(-10*(x-0.5)**2)
        # this is a little tricky.  the local vector contains ghost
        # points, which represents data that comes in from the global
        # vector when we are coordinating.  On the other hand, the
        # global vector only ever contains the data we own.  So when
        # we set initial conditions, we do it on the global vector,
        # then scatter to the local  
        fg.setArray(q)
        da.globalToLocal(fg,fl)

        # this should dump the properly set initial conditions
        fg.view()
        
        for n in range(N+1):
            # grab the working array out of the local vector
            # (including ghost points)
            q = fl.getArray()

            # operate on the local data
            q[1:]=q[1:]-cfl*(q[1:]-q[:-1])

            # restore the working array
            fl.setArray(q)

            # this is a local update, the local array in the local
            # vector is copied into the corresponding local array in
            # the global vector, ghost points are discarded 
            da.localToGlobal(fl,fg)
            
            fg.view()
            # this is a global update coordinated over all processes.
            # The local array in the global vector is copied into the
            # corresponding local array in the local vector.
            # In addition, the ghost values needed to operate locally
            # are sent over MPI to the correct positions in the local
            # array in the local vector.
            da.globalToLocal(fg,fl)

    def run(self):
        OptDB = PETSc.Options()
        M = OptDB.getInt('M', 16)
        cfl = OptDB.getReal('cfl',0.95)
        T = OptDB.getReal('T',2.)
        self.advection1D(M, cfl, T)
        print('Done')
        return
          
if __name__ == '__main__':
    PetCLAW().run()
          

    
    



"Runs the test problem."

def test_cubic():
    import numpy as np
    from . import cubic

    claw = cubic.setup(solver_type="sharpclaw")
    # For the classic solver with solver.limiters = pyclaw.limiters.tvd.vanleer,
    # nonclassical solutions occur...
    claw.run()
    sol = claw.solution.state.get_q_global()

    if sol is not None:
        xc = claw.solution.state.grid.x.centers
        qL = 4.0
        qR = -2.0
        x0 = -0.5 + (qL**3 - qR**3) / (qL - qR) * 0.2
        expected_sol = (xc < x0) * qL + (xc >= x0) * qR
        test_err = np.linalg.norm(expected_sol - sol, ord=1) / len(xc)
        print(test_err)
        assert test_err < 1.e-3


if __name__=="__main__":
    import nose
    nose.main()



"""
Some tools for making 1D grids.

make_mapc2p - based on a file celledges.data with cell edges, create a
    mapc2p file making 0 <= xc <= 1 to physical cell edges.

make_celledges_cfl - create a celledges.data file with celledges chosen so
    that the Courant number is nearly 1 in each cell in the ocean.

make_pwlin_topo_fcn - take a list of (x,z) pairs and create a function that
    is the piecewise linear interpolant through these points.

"""

from pylab import *
from scipy.interpolate import interp1d
import numpy as np
import os


def make_mapc2p(fname_celledges='celledges.data'):
    """
    Create a mapc2p function that maps computational cell edges xc
    with 0 <= xc <= 1 to the physical cell edges.  The physical
    cell edges should be in the file fname_celledges, 
    starting in the second row (following mx_edge, the number of edges).

    Returns the mapc2p function and also mx_edge, xp_edge, which may be
    needed for other purposes.
    """

    path = os.path.abspath(fname_celledges)
    d = np.loadtxt(path, skiprows=1) 
    mx_edge = d.shape[0]

    print('make_mapc2p: Using %i cell edge values from %s' % (d.shape[0], path))

    xc_edge = np.linspace(0,1,mx_edge)  # assumes xlower=0, xupper=1 in setrun
    xp_edge = d[:,0]
    mapc2p = interp1d(xc_edge, xp_edge, kind='linear')

    return mapc2p, mx_edge, xp_edge


def make_pwlin_topo_fcn(xzpairs):
    """
    Input: xzpairs should be a list of tuples (xi,zi).
    Output: a function that is the piecwise linear interpolant.
    """

    xi = array([xz[0] for xz in xzpairs])
    zi = array([xz[1] for xz in xzpairs])
    z_fcn = interp1d(xi, zi, kind='linear', bounds_error=False, 
                     fill_value='extrapolate')
    return z_fcn


def make_celledges_cfl(xlower, xupper, mx, topo_fcn, hmin,
                       fname='celledges.data', plot_topo=False):

    grav = 9.81

    cmin = sqrt(grav*hmin)

    def c(x):
        z = topo_fcn(x)
        h = where(-z > hmin, -z, hmin)
        c = sqrt(grav*h)
        return c

    xunif = linspace(xlower, xupper, 2*mx)
    cunif = c(xunif)
    csum = cumsum(1./cunif)
    csum = csum - csum[0]

    csum = csum / csum[-1]
    cinv = interp1d(csum, xunif)

    xc = linspace(0, 1, mx+1)   # computational grid
    xp = cinv(xc)
    z = topo_fcn(xp)
    dxp = diff(xp)
    
    if plot_topo:
        figure(97, figsize=(6,8))
        clf()
        subplot(311)
        #plot(csum, xunif, 'b')
        plot(xunif, csum, 'b')
        ylabel('computational coordinate xc')
        grid(True)
        axis([xlower,xupper,0,1])
        title('inverse of mapc2p function')

        subplot(312)
        xcell = 0.5*(xp[1:] + xp[:-1])
        plot(xcell, dxp, 'b')
        #xlabel('physical coordinate xp')
        ylabel('delta x')
        grid(True)
        xlim(xlower,xupper)
        title('Mesh width')
        
        subplot(313)
        #xcell = 0.5*(xp[1:] + xp[:-1])
        dxratio = dxp[1:]/dxp[:-1]
        print('dx ratio between adjacent cells varies between %.4f and %.4f' \
            % (dxratio.min(), dxratio.max()))
        plot(xcell[1:], dxratio, 'b')
        xlabel('physical coordinate xp')
        ylabel('delta x ratio')
        grid(True)
        xlim(xlower,xupper)
        title('Mesh width ratio between adjacent cells')
        tight_layout()
        
        png_fname = 'cellmap.png'
        savefig(png_fname)
        print("Created ",png_fname)

    with open(fname,'w') as f:
        f.write('%i   # number of cell edges\n' % (mx+1))

        for i in range(mx+1):
            f.write('%15.8f %15.8f\n' % (xp[i],z[i]))
        f.close()

    print("Created %s, containing %i cell edges" % (fname,mx+1))
    print("Min dx = %g, Max dx = %g" % (dxp.min(),dxp.max()))

    if plot_topo:
        figure(99, figsize=(8,4))
        clf()
        fill_between(xp,where(z<0,z,nan),0.,color=[.5,.5,1])
        plot(xp,z,'g')
        xlim(xlower,xupper)
        zmax = max(z.max(), 0)
        zmargin = 0.1*(zmax-z.min())
        ylim(z.min()-zmargin,zmax+zmargin)
        grid(True)
        title('Topography')
        png_fname = 'topo.png'
        savefig(png_fname)
        print("Created ",png_fname)

    return xp,z




# Script used to get rid of maxmx and maxmy dependencies when converting
# from 4.x to 5.0 form.  Executed in library and application directories.

#
# Fix a set of target files in directory tree rootdir by replacing
# oldpat with newpat. 
#
# Now supports wildcards in list of targetfiles.
#

import os,sys,glob

rootdir = '.'
targetfiles = ['*.f*']

oldpat_list = ["1-mbc:maxmx", "1-mbc:maxmy", "maxmx,maxmy,"]
newpat_list = ["1-mbc:mx",    "1-mbc:my",    ""]

for oldpat,newpat in zip(oldpat_list, newpat_list):
    print("============================================")
    print('Replacing "%s" with "%s"' % (oldpat,newpat))
    print("============================================")
    for (dirpath, subdirs, files) in os.walk(rootdir):
        currentdir = os.path.abspath(os.getcwd())
        os.chdir(os.path.abspath(dirpath))
        tfiles = []
        for fpat in targetfiles:
            for f in glob.glob(fpat):
                tfiles.append(f)
        for file in tfiles:

            infile = open(file,'r')
            lines = infile.read()
            infile.close()

            if lines.find(oldpat) > -1:
                lines = lines.replace(oldpat, newpat)
                print("Fixed file   ",dirpath + '/' + file)
            else:
                print("No change to ",dirpath + '/' + file)

            outfile = open(file,'w')
            outfile.write(lines)
            outfile.close()

        os.chdir(currentdir)




from __future__ import absolute_import
from clawpack import pyclaw
from clawpack import petclaw
import os

class PetClawIOTest(pyclaw.IOTest):
    @property
    def solution(self):
        return petclaw.Solution()

    @property
    def file_formats(self):
        return ['hdf5']

    @property
    def this_dir(self):
        return os.path.dirname(os.path.abspath(__file__))

    @property
    def test_data_dir(self):
        return os.path.join(self.this_dir, '../../pyclaw/tests/test_data')

    def test_io_from_binary(self):
        return

    def test_io_to_vtk(self):
        return # this test is not valid for petclaw b/c can't read binary.
        # note that petclaw can write VTK through the PETSC vtk writing
        # capabilities. 



from . import advection_1d
import numpy as np
from clawpack.pyclaw.util import check_diff
import os

def error(**kwargs):
    """
    Compute difference between initial and final solutions.
    This should vanish due to the periodic boundary conditions
    """

    claw = advection_1d.setup(outdir=None,**kwargs)
    claw.run()

    # tests are done across the entire domain of q normally
    q0 = claw.frames[0].state.get_q_global()
    qfinal = claw.frames[claw.num_output_times].state.get_q_global()

    q0 = q0.reshape([-1])
    qfinal = qfinal.reshape([-1])
    dx = claw.solution.domain.grid.delta[0]
    diff = dx*np.sum(np.abs(qfinal-q0))
    return diff


class TestAdvection1D:
    def test_python_classic(self):
        assert abs(error(kernel_language='Python',solver_type='classic')-3.203924e-04)<1e-4

    def test_fortran_classic(self):
        assert abs(error(kernel_language='Fortran',solver_type='classic')-3.203924e-04)<1e-4

    def test_python_sharpclaw(self):
        assert abs(error(kernel_language='Python',solver_type='sharpclaw')-1.163605e-05)<1e-4

    def test_fortran_sharpclaw(self):
        assert abs(error(kernel_language='Fortran',solver_type='sharpclaw')-1.163605e-05)<1e-4

    def test_sharpclaw_multistep(self):
        assert abs(error(kernel_language='Fortran',solver_type='sharpclaw',time_integrator='SSPLMMk3')-1.500727e-05)<1e-4

    def test_weno17(self):
        assert abs(error(kernel_language='Fortran',solver_type='sharpclaw',weno_order=17)-7.489618e-06)<1e-4




import ctypes
import os.path
import numpy as np

from clawpack.pyclaw import ClawSolver1D

class iso_c_step1():

    def __init__(self):
        this_path = os.path.dirname(__file__)
        self._dll = ctypes.CDLL(os.path.join(this_path, 'iso_c_classic1.so'))

        # work arrays
        self.allocated = False
        self._f = None
        self._wave = None
        self._s = None
        self._amdq = None
        self._apdq = None
        self._dtdx = None

    def allocate(self, mx, num_eqn, num_ghost, num_waves):

        # these are work arrays, so order doesn't really matter
        self._f = np.empty((2*num_ghost + mx, num_eqn), dtype=np.double)
        self._wave = np.empty((num_eqn, num_waves, 2*num_ghost + mx),
                              dtype=np.double)
        self._s = np.empty((num_waves, 2*num_ghost+mx), dtype=np.double)
        self._amdq = np.empty((num_eqn, 2*num_ghost + mx), dtype=np.double)
        self._apdq = np.empty((num_eqn, 2*num_ghost + mx), dtype=np.double)
        self._dtdx = np.empty((2*num_ghost + mx), dtype=np.double)
        self.allocated = True

    def step1(self,
              py_num_ghost,
              py_mx,
              qbc,
              auxbc,
              py_dx,
              py_dt,
              method,
              mthlim,
              fwave,
              rp):
        r"""
        Take one time step on the homogeneous hyperbolic system.

        This function directly wraps the Clawpack step1 call, and is responsible
        for translating Pythonic data structures into their C/Fortran
        equivalents.
        """

        from ctypes import c_int, c_double, c_bool, byref

        # a real solver object would be caching/verifying these values, this is
        # just scaffolding

        py_num_eqn, mxbc = qbc.shape
        num_eqn = c_int(py_num_eqn)

        mthlim = np.asarray(mthlim, dtype=np.int)
        py_num_waves = mthlim.shape[0]
        num_waves = c_int(py_num_waves)

        py_num_aux, mxauxbc = auxbc.shape
        num_aux = c_int(py_num_aux)

        cfl = c_double()
        use_fwave = c_bool(False)

        num_ghost = c_int(py_num_ghost)
        mx = c_int(py_mx)

        dx = c_double(py_dx)
        dt = c_double(py_dt)

        if not self.allocated:
            self.allocate(py_mx, py_num_eqn, py_num_ghost, py_num_waves)


        def to_double_ref(nparray):
            return nparray.ctypes.data_as(ctypes.POINTER(ctypes.c_double))


        def to_int_ref(nparray):
            return nparray.ctypes.data_as(ctypes.POINTER(ctypes.c_int))


        self._dll.step1_c(byref(num_eqn),
                          byref(num_waves),
                          byref(num_ghost),
                          byref(num_aux),
                          byref(mx),
                          to_double_ref(qbc),
                          to_double_ref(auxbc),
                          byref(dx),
                          byref(dt),
                          to_int_ref(method),
                          to_int_ref(mthlim),
                          byref(cfl),
                          to_double_ref(self._f),
                          to_double_ref(self._wave),
                          to_double_ref(self._s),
                          to_double_ref(self._amdq),
                          to_double_ref(self._apdq),
                          to_double_ref(self._dtdx),
                          byref(use_fwave),
                          byref(rp._rp),
                          byref(rp.context))


        return qbc, cfl.value


class ISO_C_ClawSolver1D(ClawSolver1D):

    def __init__(self,riemann_solver=None,claw_package=None):
        r"""
        Create 1d ISO C Clawpack solver

        See :class:`ClawSolver1D` for more info.
        """

        self.iso_c_step1 = iso_c_step1()


        super(ISO_C_ClawSolver1D,self).__init__(riemann_solver,claw_package)


    def step_hyperbolic(self,solution):
        r"""
        Take one time step on the homogeneous hyperbolic system.

        :Input:
        - *solution* - (:class:`~pyclaw.solution.Solution`) Solution that
        will be evolved
        """

        state = solution.states[0]
        grid = state.grid

        self._apply_bcs(state)

        num_eqn,num_ghost = state.num_eqn,self.num_ghost

        mx = grid.num_cells[0]
        dx,dt = grid.delta[0],self.dt
        dtdx = np.zeros( (mx+2*num_ghost) ) + dt/dx

        self.qbc,cfl = self.iso_c_step1.step1(num_ghost, mx, self.qbc,
                                              self.auxbc, dx, dt, self._method,
                                              self._mthlim,self.fwave,
                                              self.rp)
        self.cfl.update_global_max(cfl)
        state.set_q_from_qbc(num_ghost,self.qbc)


        if state.num_aux > 0:
            state.set_aux_from_auxbc(num_ghost,self.auxbc)



#!/usr/bin/env python
# encoding: utf-8

"""
2D Euler Riemann problem
================================

Solve the Euler equations of compressible fluid dynamics:

.. math::
    \rho_t + (\rho u)_x + (\rho v)_y & = 0 \\
    (\rho u)_t + (\rho u^2 + p)_x + (\rho uv)_y & = 0 \\
    (\rho v)_t + (\rho uv)_x + (\rho v^2 + p)_y & = 0 \\
    E_t + (u (E + p) )_x + (v (E + p))_y & = 0.

Here :math:`\rho` is the density, (u,v) is the velocity, and E is the total energy.
The initial condition is one of the 2D Riemann problems from the paper of
Liska and Wendroff.

This example shows how to use VisClaw's Iplot class for simple interactive plotting.
"""
from clawpack import pyclaw
from clawpack import riemann
from clawpack.riemann.euler_4wave_2D_constants import density, x_momentum, y_momentum, \
        energy, num_eqn

def load_frame(frame_number):
    from clawpack.pyclaw import Solution

    return Solution(frame_number)

def plot_frame(frame):
    import matplotlib.pyplot as plt
    q = frame.q
    x, y = frame.state.grid.c_centers
    plt.pcolormesh(x, y, q[density,...])

def plot_results():
    from clawpack.visclaw import iplot
    ip = iplot.Iplot(load_frame,plot_frame)
    ip.plotloop()

solver = pyclaw.ClawSolver2D(riemann.euler_4wave_2D)
solver.all_bcs = pyclaw.BC.extrap

domain = pyclaw.Domain([0.,0.],[1.,1.],[200,200])
solution = pyclaw.Solution(num_eqn,domain)
gamma = 1.4
solution.problem_data['gamma']  = gamma

# Set initial data
xx,yy = domain.grid.p_centers
l = xx<0.5; r = xx>=0.5; b = yy<0.5; t = yy>=0.5
solution.q[density,...] = 2.*l*t + 1.*l*b + 1.*r*t + 3.*r*b
solution.q[x_momentum,...] = 0.75*t - 0.75*b
solution.q[y_momentum,...] = 0.5*l  - 0.5*r
solution.q[energy,...] = 0.5*solution.q[density,...]*(solution.q[x_momentum,...]**2+solution.q[y_momentum,...]**2) + 1./(gamma-1.)

claw = pyclaw.Controller()
claw.tfinal = 0.3
claw.solution = solution
claw.solver = solver

status = claw.run()

#plot_results()



#!/usr/bin/env python

r"""Regression tests.  Execute via:
    python regression_tests.py
to test, or
    python regression_tests.py True
to create new regression data for archiving.
"""

from __future__ import absolute_import
import os
import sys
import unittest
import shutil

import numpy

import clawpack.geoclaw.test as test
import clawpack.geoclaw.topotools as topotools
import clawpack.geoclaw.dtopotools as dtopotools

class DTopoTests(test.GeoClawRegressionTest):
    
    def setUp(self):

        super(DTopoTests, self).setUp()

        # Make topography
        h0 = 1000.0
        topo_func = lambda x,y: -h0*(1 + 0.5 * numpy.cos(x - y))
        topo = topotools.Topography(topo_func=topo_func)
        topo.topo_type = 2
        topo.x = numpy.linspace(-10.0, 10.0, 201)
        topo.y = numpy.linspace(-10.0, 10.0, 201)
        topo.write(os.path.join(self.temp_path, "topo1.topotype2"), \
                topo_type=2, Z_format="%22.15e")

        h0 = 1000.0
        topo_func = lambda x,y: -h0*(1. + numpy.exp(x+y))
        topo = topotools.Topography(topo_func=topo_func)
        topo.topo_type = 2
        topo.x = numpy.linspace(-0.5, -0.3, 21)
        topo.y = numpy.linspace(-0.1, 0.4, 51)
        topo.write(os.path.join(self.temp_path, "topo2.topotype2"), \
                topo_type=2, Z_format="%22.15e")

        # Make dtopography
        subfault_path = os.path.join(self.test_path, "dtopo1.csv")
        input_units = {'slip': 'm', 'depth': 'km', 'length': 'km', 'width': 'km'}
        fault = dtopotools.CSVFault()
        fault.read(subfault_path, input_units=input_units, 
                coordinate_specification="top center")
        fault.rupture_type = 'dynamic'
        times = numpy.linspace(0.0, 1.0, 25)
        x = numpy.linspace(-0.4,0.6,151)
        y = numpy.linspace(-0.4,0.4,121)
        dtopo = fault.create_dtopography(x,y,times=times)
        dtopo.write(os.path.join(self.temp_path, "dtopo1.tt3"), dtopo_type=3)

        subfault_path = os.path.join(self.test_path, "dtopo2.csv")
        input_units = {'slip': 'm', 'depth': 'km', 'length': 'km', 'width': 'km'}
        fault = dtopotools.CSVFault()
        fault.read(subfault_path, input_units=input_units, 
                    coordinate_specification="top center")
        fault.rupture_type = 'dynamic'
        times = numpy.linspace(0.5, 1.2, 25)
        x = numpy.linspace(-0.9,0.1,201)
        y = numpy.linspace(-0.4,0.4,161)
        dtopo = fault.create_dtopography(x,y,times=times)
        dtopo.write(os.path.join(self.temp_path, "dtopo2.tt3"), dtopo_type=3)

        # copy existing file:
        shutil.copy(os.path.join(self.test_path, "dtopo3.tt1"),
                                 self.temp_path)


    def check_gauges(self, save=False, gauge_id=1, regression_gauge_id=1, 
                     indices=[0], rtol=1e-14, atol=1e-8, tolerance=None):

        r"""Basic test to assert gauge equality

        :Input:
         - *save* (bool) - If *True* will save the output from this test to 
           the file *regresion_data.txt*.  Default is *False*.
         - *gauge_id* (int) - The gauge to test.
         - *regression_gauge_id* (int) - The gauge to test against.
         - *indices* (tuple) - Contains indices to compare in the gague 
           comparison.  Defaults to *(0)*.
         - *rtol* (float) - Relative tolerance used in the comparison, default 
           is *1e-14*.  Note that the old *tolerance* input is now synonymous 
           with this parameter.
         - *atol* (float) - Absolute tolerance used in the comparison, default
           is *1e-08*.
        """

        import clawpack.pyclaw.gauges as gauges
        import clawpack.clawutil.claw_git_status as claw_git_status

        if isinstance(tolerance, float):
            rtol = tolerance

        if not(isinstance(indices, tuple) or isinstance(indices, list)):
            indices = tuple(indices)

        # Get gauge data
        gauge = gauges.GaugeSolution(gauge_id, path=self.temp_path)

        # Get regression comparison data
        regression_data_path = os.path.join(self.test_path, "regression_data")
        if save:
            gauge_file_name = "gauge%s.txt" % str(regression_gauge_id).zfill(5)
            shutil.copy(os.path.join(self.temp_path, gauge_file_name), 
                                                           regression_data_path)
            claw_git_status.make_git_status_file(outdir=regression_data_path)

        regression_gauge = gauges.GaugeSolution(regression_gauge_id,
                                                path=regression_data_path)

        # Compare data
        try:
            for n in indices:
                numpy.testing.assert_allclose(gauge.q[n, :],
                                              regression_gauge.q[n, :], 
                                              rtol=rtol, atol=atol, 
                                              verbose=False)
        except AssertionError as e:
            err_msg = "\n".join((e.args[0], 
                                "Gauge Match Failed for gauge = %s" % gauge_id))
            err_msg = "\n".join((err_msg, "  failures in fields:"))
            failure_indices = []
            for n in indices:
                if ~numpy.allclose(gauge.q[n, :], regression_gauge.q[n, :], 
                                                          rtol=rtol, atol=atol):
                    failure_indices.append(str(n))
            index_str = ", ".join(failure_indices)
            raise AssertionError(" ".join((err_msg, index_str)))



    def runTest(self, save=False, indices=(2, 3)):
        r"""DTopography basic regression test

        :Input:
         - *save* (bool) - If *True* will save the output from this test to 
           the file *regresion_data.txt*.  Passed to *check_gauges*.  Default is
           *False*.
         - *indices* (tuple) - Contains indices to compare in the gague 
           comparison and passed to *check_gauges*.  Defaults to *(2, 3)*.

        """

        # Write out data files
        self.load_rundata()
        self.write_rundata_objects()

        # Run code
        self.run_code()

        # Perform tests
        self.check_gauges(save=save, gauge_id=1, regression_gauge_id=1,
                          indices=(2, 3))
        print('gauge 1 ascii agrees')
        self.check_gauges(save=save, gauge_id=2, regression_gauge_id=1,
                          indices=(2, 3), rtol=1e-6, atol=1e-6)
        print('gauge 2 binary agrees')

        # If we have gotten here then we do not need to copy the run results
        self.success = True


if __name__=="__main__":
    if len(sys.argv) > 1:
        if bool(sys.argv[1]):
            # Fake the setup and save out output
            test = DTopoTests()
            try:
                test.setUp()
                test.runTest(save=True)
            finally:
                test.tearDown()
            sys.exit(0)
    unittest.main()




#!/usr/bin/env python
from __future__ import absolute_import
from __future__ import print_function
from petsc4py import PETSc
from six.moves import range

class PetCLAW:
  def advection1D(self, N):
    '''David: If you put in the linear algebra that you need (comments), I will move it to code'''
    da = PETSc.DA().create([N])
    da.view()
    f = da.createGlobalVector()
    f.view()
    a = f.getArray()
    for i in range(f.getSize()):
      a[i] = i*i
    f.view()
    return

  def run(self):
    self.advection1D(10)
    print('Done')
    return

if __name__ == '__main__':
  PetCLAW().run()



r"""
Execute nosetests in all subdirectories, to run a series of quick
regression tests.

Sends output and result/errors to separate files to simplify checking
results and looking for errors.
"""

from __future__ import absolute_import
import os
import glob

import numpy

import clawpack.clawutil.test
import clawpack.pyclaw.util

# Clean library files whenever this module is used
if "CLAW" in os.environ:
    CLAW = os.environ["CLAW"]
else:
    raise ValueError("Need to set CLAW environment variable.")

for lib_path in [os.path.join(CLAW,"amrclaw","src","2d"),
                 os.path.join(CLAW,"geoclaw","src","2d","shallow"),
                 os.path.join(CLAW,"geoclaw","src","2d","shallow","multilayer"),
                 os.path.join(CLAW,"geoclaw","src","2d","shallow","surge")]:
    for path in glob.glob(os.path.join(lib_path,"*.o")):
        os.remove(path)
    for path in glob.glob(os.path.join(lib_path,"*.mod")):
        os.remove(path)


class GeoClawRegressionTest(clawpack.clawutil.test.ClawpackRegressionTest):

    r"""Base GeoClaw regression test setup derived from ClawpackRegressionTest

    """

    __doc__ += clawpack.pyclaw.util.add_parent_doc(
                                  clawpack.clawutil.test.ClawpackRegressionTest)


    def build_executable(self, executable_name="xgeoclaw"):
        r"""Build executable by running `make .exe` in test directory.

        Moves the resulting executable to the temporary directory.


        """

        super(GeoClawRegressionTest, self).build_executable(
                                                executable_name=executable_name)


    def check_fgmax(self, fgno=1, save=False):
        r"""Basic test to assert fgmax equality
        Currently just records sum of fg.h and of fg.s.

        :Input:
         - *save* (bool) - If *True* will save the output from this test to 
           the file *regresion_data.txt*.  Default is *False*.
        """

        from clawpack.geoclaw import fgmax_tools

        fg = fgmax_tools.FGmaxGrid()
        fname = os.path.join(self.temp_path, 'fgmax_grids.data')
        fg.read_fgmax_grids_data(fgno, fname)
        fg.read_output(outdir=self.temp_path)

        data_sum = numpy.array([fg.h.sum(), fg.s.sum()])

        # Get (and save) regression comparison data
        regression_data_file = os.path.join(self.test_path, "regression_data",
                "regression_data_fgmax.txt")
        if save:
            numpy.savetxt(regression_data_file, data_sum)
        regression_sum = numpy.loadtxt(regression_data_file)

        # Compare data
        tolerance = 1e-14
        assert numpy.allclose(data_sum, regression_sum, tolerance), \
                "\n data: %s, \n expected: %s" % (data_sum, regression_sum)



#!/usr/bin/env python
# encoding: utf-8
r"""
Two-dimensional acoustics
=========================

Solve the (linear) acoustics equations:

.. math:: 
    p_t + K (u_x + v_y) & = 0 \\ 
    u_t + p_x / \rho & = 0 \\
    v_t + p_y / \rho & = 0.

Here p is the pressure, (u,v) is the velocity, K is the bulk modulus,
and :math:`\rho` is the density.
"""
from clawpack import riemann
import numpy as np

def setup(kernel_language='Fortran', use_petsc=False, outdir='./_output', 
              solver_type='classic', time_integrator='SSP104', ptwise=False,
              disable_output=False):
    """
    Example python script for solving the 2d acoustics equations.
    """
    if use_petsc:
        from clawpack import petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if solver_type == 'classic':
        if ptwise:
            solver = pyclaw.ClawSolver2D(riemann.acoustics_2D_ptwise)
        else:
            solver = pyclaw.ClawSolver2D(riemann.acoustics_2D)
        solver.dimensional_split=True
        solver.cfl_max = 0.5
        solver.cfl_desired = 0.45
        solver.limiters = pyclaw.limiters.tvd.MC
    elif solver_type=='sharpclaw':
        solver=pyclaw.SharpClawSolver2D(riemann.acoustics_2D)
        solver.time_integrator=time_integrator
        if solver.time_integrator=='SSP104':
            solver.cfl_max = 0.5
            solver.cfl_desired = 0.45
        elif solver.time_integrator=='SSPLMMk2':
            solver.lmm_steps = 3
            solver.lim_type = 2
            solver.cfl_max = 0.25
            solver.cfl_desired = 0.24
        else:
            raise Exception('CFL desired and CFL max have not been provided for the particular time integrator.')
    
    solver.bc_lower[0]=pyclaw.BC.extrap
    solver.bc_upper[0]=pyclaw.BC.extrap
    solver.bc_lower[1]=pyclaw.BC.extrap
    solver.bc_upper[1]=pyclaw.BC.extrap

    mx=100; my=100
    x = pyclaw.Dimension(-1.0,1.0,mx,name='x')
    y = pyclaw.Dimension(-1.0,1.0,my,name='y')
    domain = pyclaw.Domain([x,y])

    num_eqn = 3
    state = pyclaw.State(domain,num_eqn)

    rho  = 1.0  # Material density
    bulk = 4.0  # Material bulk modulus
    cc = np.sqrt(bulk/rho)  # sound speed
    zz = rho*cc             # impedance
    state.problem_data['rho']= rho
    state.problem_data['bulk']=bulk
    state.problem_data['zz']= zz
    state.problem_data['cc']=cc

    solver.dt_initial=np.min(domain.grid.delta)/state.problem_data['cc']*solver.cfl_desired

    qinit(state)

    claw = pyclaw.Controller()
    claw.keep_copy = True
    if disable_output:
        claw.output_format = None
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    claw.outdir = outdir
    claw.num_output_times = 10
    claw.tfinal = 0.12
    claw.setplot = setplot

    return claw

def qinit(state,width=0.2):
    X, Y = state.grid.p_centers
    r = np.sqrt(X**2 + Y**2)

    state.q[0,:,:] = (np.abs(r-0.5)<=width)*(1.+np.cos(np.pi*(r-0.5)/width))
    state.q[1,:,:] = 0.
    state.q[2,:,:] = 0.


def setplot(plotdata):
    """ 
    Plot output with VisClaw.
    This example demonstrates how to plot a 1D projection from 2D data.
    """ 

    from clawpack.visclaw import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data
    
    # Figure for pressure
    plotfigure = plotdata.new_plotfigure(name='Pressure', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Pressure'
    plotaxes.scaled = True      # so aspect ratio is 1

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 0
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = True

    # Figure for scatter plot
    plotfigure = plotdata.new_plotfigure(name='scatter', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Scatter plot'

    # Set up for item on these axes: scatter of 2d data
    plotitem = plotaxes.new_plotitem(plot_type='1d_from_2d_data')
    
    def p_vs_r(current_data):
        # Return radius of each patch cell and p value in the cell
        from pylab import sqrt
        x = current_data.x
        y = current_data.y
        r = sqrt(x**2 + y**2)
        q = current_data.q
        p = q[0,:,:]
        return r,p

    plotitem.map_2d_to_1d = p_vs_r
    plotitem.plot_var = 0
    plotitem.plotstyle = 'ob'
    
    return plotdata

    
if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



#!/usr/bin/env python
# encoding: utf-8
r"""
2D shallow water: flow over a sill
==================================

Solve the 2D shallow water equations with
variable bathymetry:

.. :math:
    h_t + (hu)_x + (hv)_y & = 0 \\
    (hu)_t + (hu^2 + \frac{1}{2}gh^2)_x + (huv)_y & = -g h b_x \\
    (hv)_t + (huv)_x + (hv^2 + \frac{1}{2}gh^2)_y & = -g h b_y.

The bathymetry contains a gaussian hump.
are outflow.
"""

from clawpack import riemann
from clawpack import pyclaw
from clawpack.riemann.shallow_roe_with_efix_2D_constants import depth, x_momentum, y_momentum, num_eqn
import numpy as np

amplitude = 0.1  # Height of incoming wave
t_bdy = 5.       # Stop sending in waves at this time

def bathymetry(x,y):
    r2 = (x-1.)**2 + (y-0.5)**2
    return 0.8*np.exp(-10*r2)

def wave_maker_bc(state,dim,t,qbc,auxbc,num_ghost):
    "Generate waves at left boundary as if there were a moving wall there."
    if dim.on_lower_boundary:
        qbc[0,:num_ghost,:]=qbc[0,num_ghost,:]
        t=state.t
        if t <= t_bdy:
            vwall = amplitude*(np.sin(t*np.pi/1.5))
        else:
            vwall=0.
        for ibc in range(num_ghost-1):
            qbc[1,num_ghost-ibc-1,:] = 2*vwall - qbc[1,num_ghost+ibc,:]


def setup(kernel_language='Fortran', solver_type='classic', use_petsc=False,
          outdir='./_output'):

    solver = pyclaw.ClawSolver2D(riemann.shallow_bathymetry_fwave_2D)
    solver.dimensional_split = 1  # No transverse solver available

    solver.bc_lower[0] = pyclaw.BC.custom
    solver.user_bc_lower = wave_maker_bc
    solver.bc_upper[0] = pyclaw.BC.extrap
    solver.bc_lower[1] = pyclaw.BC.wall
    solver.bc_upper[1] = pyclaw.BC.wall

    solver.aux_bc_lower[0] = pyclaw.BC.extrap
    solver.aux_bc_upper[0] = pyclaw.BC.extrap
    solver.aux_bc_lower[1] = pyclaw.BC.extrap
    solver.aux_bc_upper[1] = pyclaw.BC.extrap

    my = 20
    mx = 4*my
    x = pyclaw.Dimension(0.,4.,mx,name='x')
    y = pyclaw.Dimension(0,1, my,name='y')
    domain = pyclaw.Domain([x,y])
    state = pyclaw.State(domain,num_eqn,num_aux=1)

    X, Y = state.p_centers
    state.aux[0,:,:] = bathymetry(X,Y)

    state.q[depth,:,:] = 1. - state.aux[0,:,:]
    state.q[x_momentum,:,:] = 0.
    state.q[y_momentum,:,:] = 0.

    state.problem_data['grav'] = 1.0
    state.problem_data['dry_tolerance'] = 1.e-3
    state.problem_data['sea_level'] = 0.

    claw = pyclaw.Controller()
    claw.tfinal = 10
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    claw.num_output_times = 40
    claw.setplot = setplot
    claw.keep_copy = True

    return claw

def surface_height(current_data):
    h = current_data.q[0,:,:]
    b = bathymetry(current_data.x,current_data.y)
    return h+b

def setplot(plotdata):
    from clawpack.visclaw import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data

    # Figure for q[0]
    plotfigure = plotdata.new_plotfigure(name='Water height', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Water height'
    plotaxes.scaled = False

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = surface_height
    plotitem.pcolor_cmap = colormaps.red_yellow_blue
    plotitem.pcolor_cmin = 0.9
    plotitem.pcolor_cmax = 1.1
    plotitem.add_colorbar = True

    return plotdata


if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)




"""
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.

"""

import numpy as np
import matplotlib.pyplot as plt

from clawpack.geoclaw import topotools
import os,sys

cmax = 0.5
cmin = -cmax

cmax_land = 2.

#--------------------------
def setplot(plotdata=None):
#--------------------------

    """
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.

    """


    from clawpack.visclaw import colormaps, geoplot
    from numpy import linspace

    if plotdata is None:
        from clawpack.visclaw.data import ClawPlotData
        plotdata = ClawPlotData()


    plotdata.clearfigures()  # clear any old figures,axes,items data
    plotdata.format = 'binary'



    def timeformat(t):
        from numpy import mod
        hours = int(t/3600.)
        tmin = mod(t,3600.)
        min = int(tmin/60.)
        sec = int(mod(tmin,60.))
        timestr = '%s:%s:%s' % (hours,str(min).zfill(2),str(sec).zfill(2))
        return timestr

    def title_hours(current_data):
        from pylab import title
        t = current_data.t
        timestr = timeformat(t)
        title('%s after earthquake' % timestr)


    #-----------------------------------------
    # Figure for surface
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Computational domain', figno=0)
    plotfigure.kwargs = {'figsize':(8,7)}
    plotfigure.show = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('pcolor')
    plotaxes.title = 'Surface'
    plotaxes.scaled = True


    def aa(current_data):
        from pylab import ticklabel_format, xticks, gca, cos, pi, savefig
        gca().set_aspect(1.)
        title_hours(current_data)
        ticklabel_format(useOffset=False)
        xticks(rotation=20)


    plotaxes.afteraxes = aa


    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    #plotitem.plot_var = geoplot.surface
    plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = cmin
    plotitem.pcolor_cmax = cmax
    plotitem.add_colorbar = True
    plotitem.colorbar_shrink = 0.7
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.amr_patchedges_show = [0,0,0,0]
    plotitem.amr_data_show = [1,1,1,1,1,0,0]

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = cmax_land
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [0]
    plotitem.amr_patchedges_show = [0,0,0,0]
    plotitem.amr_data_show = [1,1,1,1,1,0,0]

    # add contour lines of bathy if desired:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.contour_levels = linspace(-3000,-3000,1)
    plotitem.amr_contour_colors = ['y']  # color on each level
    plotitem.kwargs = {'linestyles':'solid','linewidths':2}
    plotitem.amr_contour_show = [1,0,0]
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0

    #-----------------------------------------
    # Figure for coastal area
    #-----------------------------------------
    x1,x2,y1,y2 = [-0.005, 0.016, -0.01, 0.01]

    plotfigure = plotdata.new_plotfigure(name="coastal area", figno=11)
    plotfigure.show = True
    plotfigure.kwargs = {'figsize': (6,7)}

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.scaled = False

    plotaxes.xlimits = [x1, x2]
    plotaxes.ylimits = [y1, y2]

    def aa_withbox(current_data):
        from pylab import plot
        x1,x2,y1,y2 = (-0.009259, 0.013796, -0.005093, 0.005000)
        if current_data.t > 5*60.:
            plot([x1,x1,x2,x2,x1], [y1,y2,y2,y1,y1], 'w--')
        aa(current_data)
        
    plotaxes.afteraxes = aa_withbox

    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.surface
    #plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = cmin
    plotitem.pcolor_cmax = cmax
    plotitem.add_colorbar = True
    plotitem.colorbar_shrink = 0.4
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 0

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = cmax_land
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [0]
    plotitem.patchedges_show = 0

    # add contour lines of bathy if desired:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    #plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.contour_levels = [-2,-1,0,1,2]
    plotitem.amr_contour_colors = ['yellow']  # color on each level
    plotitem.kwargs = {'linestyles':'solid','linewidths':1}
    plotitem.amr_contour_show = [0,0,0,1]
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0



    # Plots of timing (CPU and wall time):

    def make_timing_plots(plotdata):
        import os
        from clawpack.visclaw import plot_timing_stats
        try:
            timing_plotdir = plotdata.plotdir + '/_timing_figures'
            os.system('mkdir -p %s' % timing_plotdir)
            units = {'comptime':'minutes', 'simtime':'minutes', 'cell':'millions'}
            plot_timing_stats.make_plots(outdir=plotdata.outdir, make_pngs=True,
                                          plotdir=timing_plotdir, units=units)
            os.system('cp %s/timing.* %s' % (plotdata.outdir, timing_plotdir))
        except:
            print('*** Error making timing plots')

    otherfigure = plotdata.new_otherfigure(name='timing',
                    fname='_timing_figures/timing.html')
    otherfigure.makefig = make_timing_plots


    #-----------------------------------------

    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'        # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?
    plotdata.parallel = True                 # make multiple frame png's at once

    return plotdata
